# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .profile_type_enum import ProfileTypeEnum


@JsonMap({"id_": "id"})
class PeopleMe(BaseModel):
    """People object representing a Deel

    :param id_: Unique identifier for the user profile, defaults to None
    :type id_: float, optional
    :param user_id: Unique identifier for the user, defaults to None
    :type user_id: float, optional
    :param username: Username of the user, defaults to None
    :type username: str, optional
    :param first_name: First name of the user, defaults to None
    :type first_name: str, optional
    :param middle_name: Middle name of the user, defaults to None
    :type middle_name: str, optional
    :param last_name: Last name of the user, defaults to None
    :type last_name: str, optional
    :param full_name: Full name of the user, defaults to None
    :type full_name: str, optional
    :param email: Email address of the user, defaults to None
    :type email: str, optional
    :param avatar_url: URL of the user's avatar, defaults to None
    :type avatar_url: str, optional
    :param profile_type: profile_type, defaults to None
    :type profile_type: ProfileTypeEnum, optional
    :param slack_user_id: User's slack id, defaults to None
    :type slack_user_id: str, optional
    :param slack_team_id: User's team slack id, defaults to None
    :type slack_team_id: str, optional
    :param organization_id: Unique identifier for the organization, defaults to None
    :type organization_id: float, optional
    :param organization_name: Name of the organization, defaults to None
    :type organization_name: str, optional
    :param client_country: Country code., defaults to None
    :type client_country: str, optional
    :param contractor_country: Country code., defaults to None
    :type contractor_country: str, optional
    :param hris_enabled: HRIS feature enabled., defaults to None
    :type hris_enabled: bool, optional
    """

    def __init__(
        self,
        id_: float = None,
        user_id: float = None,
        username: str = None,
        first_name: str = None,
        middle_name: str = None,
        last_name: str = None,
        full_name: str = None,
        email: str = None,
        avatar_url: str = None,
        profile_type: ProfileTypeEnum = None,
        slack_user_id: str = None,
        slack_team_id: str = None,
        organization_id: float = None,
        organization_name: str = None,
        client_country: str = None,
        contractor_country: str = None,
        hris_enabled: bool = None,
    ):
        if id_ is not None:
            self.id_ = id_
        if user_id is not None:
            self.user_id = user_id
        if username is not None:
            self.username = username
        if first_name is not None:
            self.first_name = first_name
        if middle_name is not None:
            self.middle_name = middle_name
        if last_name is not None:
            self.last_name = last_name
        if full_name is not None:
            self.full_name = full_name
        if email is not None:
            self.email = email
        if avatar_url is not None:
            self.avatar_url = avatar_url
        if profile_type is not None:
            self.profile_type = self._enum_matching(
                profile_type, ProfileTypeEnum.list(), "profile_type"
            )
        if slack_user_id is not None:
            self.slack_user_id = slack_user_id
        if slack_team_id is not None:
            self.slack_team_id = slack_team_id
        if organization_id is not None:
            self.organization_id = organization_id
        if organization_name is not None:
            self.organization_name = organization_name
        if client_country is not None:
            self.client_country = self._pattern_matching(
                client_country, "^[A-Z]{2}$", "client_country"
            )
        if contractor_country is not None:
            self.contractor_country = self._pattern_matching(
                contractor_country, "^[A-Z]{2}$", "contractor_country"
            )
        if hris_enabled is not None:
            self.hris_enabled = hris_enabled



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .eor_entitlement_list_item import EorEntitlementListItem


@JsonMap({})
class EorEntitlements(BaseModel):
    """Time-off Entitlements for EOR contracts.

    :param entitlements: List of entitlements.
    :type entitlements: List[EorEntitlementListItem]
    """

    def __init__(self, entitlements: List[EorEntitlementListItem]):
        self.entitlements = self._define_list(entitlements, EorEntitlementListItem)



# This file was generated by liblab | https://liblab.com/

from typing import Union
from .utils.json_map import JsonMap
from .base import BaseModel
from .base import OneOfBaseModel


class AdjustmentToCreateAmountGuard(OneOfBaseModel):
    class_list = {"str": str, "float": float}


AdjustmentToCreateAmount = Union[str, float]


@JsonMap({})
class AdjustmentToCreate(BaseModel):
    """Details of adjustment to create

    :param contract_id: The identifier of the contract associated with the adjustment
    :type contract_id: str
    :param amount: Amount of adjustment.
    :type amount: AdjustmentToCreateAmount
    :param date_of_adjustment: Short date in format ISO-8601 (YYYY-MM-DD). For example 2022-12-31., defaults to None
    :type date_of_adjustment: str, optional
    :param title: Title of adjustment.
    :type title: str
    :param description: Description of adjustment.
    :type description: str
    :param cycle_reference: Cycle reference of adjustment., defaults to None
    :type cycle_reference: str, optional
    :param file: File of adjustment.
    :type file: str
    :param adjustment_category_id: Adjustment category id.
    :type adjustment_category_id: str
    :param move_next_cycle: If an adjustments can belong to another payroll cycle., defaults to None
    :type move_next_cycle: bool, optional
    :param vendor: Vendor of adjustment.
    :type vendor: str
    :param country: Country code.
    :type country: str
    """

    def __init__(
        self,
        contract_id: str,
        amount: AdjustmentToCreateAmount,
        title: str,
        description: str,
        file: str,
        adjustment_category_id: str,
        vendor: str,
        country: str,
        date_of_adjustment: str = None,
        cycle_reference: str = None,
        move_next_cycle: bool = None,
    ):
        self.contract_id = contract_id
        self.amount = AdjustmentToCreateAmountGuard.return_one_of(amount)
        if date_of_adjustment is not None:
            self.date_of_adjustment = self._pattern_matching(
                date_of_adjustment, "^\d{4}-\d{2}-\d{2}$", "date_of_adjustment"
            )
        self.title = title
        self.description = description
        if cycle_reference is not None:
            self.cycle_reference = cycle_reference
        self.file = file
        self.adjustment_category_id = adjustment_category_id
        if move_next_cycle is not None:
            self.move_next_cycle = move_next_cycle
        self.vendor = vendor
        self.country = self._pattern_matching(country, "^[A-Z]{2}$", "country")



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"id_": "id"})
class GenericResultCreatedWithIdData(BaseModel):
    """GenericResultCreatedWithIdData

    :param id_: Unique identifier of this resource., defaults to None
    :type id_: str, optional
    :param created: created
    :type created: bool
    """

    def __init__(self, created: bool, id_: str = None):
        if id_ is not None:
            self.id_ = id_
        self.created = created


@JsonMap({})
class GenericResultCreatedWithId(BaseModel):
    """GenericResultCreatedWithId

    :param data: data
    :type data: GenericResultCreatedWithIdData
    """

    def __init__(self, data: GenericResultCreatedWithIdData):
        self.data = self._define_object(data, GenericResultCreatedWithIdData)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import Union
from .base import OneOfBaseModel
from .contract_to_create_pay_as_you_go_time_based import (
    ContractToCreatePayAsYouGoTimeBased,
)
from .contract_to_create_payg_tasks import ContractToCreatePaygTasks
from .contract_to_create_payg_milestones import ContractToCreatePaygMilestones
from .contract_to_create_ongoing_time_based import ContractToCreateOngoingTimeBased


class ContractToCreateGuard(OneOfBaseModel):
    class_list = {
        "ContractToCreatePayAsYouGoTimeBased": ContractToCreatePayAsYouGoTimeBased,
        "ContractToCreatePaygTasks": ContractToCreatePaygTasks,
        "ContractToCreatePaygMilestones": ContractToCreatePaygMilestones,
        "ContractToCreateOngoingTimeBased": ContractToCreateOngoingTimeBased,
    }


ContractToCreate = Union[
    ContractToCreatePayAsYouGoTimeBased,
    ContractToCreatePaygTasks,
    ContractToCreatePaygMilestones,
    ContractToCreateOngoingTimeBased,
]



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .invoice_adjustment_status_enum import InvoiceAdjustmentStatusEnum


@JsonMap({"id_": "id"})
class OffCyclePaymentReportedBy(BaseModel):
    """OffCyclePaymentReportedBy

    :param id_: Unique identifier of this resource.
    :type id_: str
    :param full_name: Full name of the creator.
    :type full_name: str
    """

    def __init__(self, id_: str, full_name: str):
        self.id_ = id_
        self.full_name = full_name


@JsonMap({"id_": "id"})
class OffCyclePayment(BaseModel):
    """OffCyclePayment

    :param id_: Unique identifier of this resource.
    :type id_: str
    :param status: status of invoice adjustment
    :type status: InvoiceAdjustmentStatusEnum
    :param description: Description of the off-cycle payment entry.
    :type description: str
    :param date_submitted: Long date-time format following ISO-8601
    :type date_submitted: str
    :param currency_code: currency_code
    :type currency_code: str
    :param amount: Amount of off-cycle payment.
    :type amount: str
    :param created_at: Long date-time format following ISO-8601
    :type created_at: str
    :param reported_by: reported_by
    :type reported_by: OffCyclePaymentReportedBy
    """

    def __init__(
        self,
        id_: str,
        status: InvoiceAdjustmentStatusEnum,
        description: str,
        date_submitted: str,
        currency_code: str,
        amount: str,
        created_at: str,
        reported_by: OffCyclePaymentReportedBy,
    ):
        self.id_ = id_
        self.status = self._enum_matching(
            status, InvoiceAdjustmentStatusEnum.list(), "status"
        )
        self.description = description
        self.date_submitted = date_submitted
        self.currency_code = self._pattern_matching(
            currency_code, "^[A-Z]{3}$", "currency_code"
        )
        self.amount = amount
        self.created_at = created_at
        self.reported_by = self._define_object(reported_by, OffCyclePaymentReportedBy)



# This file was generated by liblab | https://liblab.com/

from requests import Response as RequestsResponse
from urllib.parse import parse_qs


class Response:
    """
    A simple HTTP response wrapper class using the requests library.

    :ivar int status_code: The status code of the HTTP response.
    :ivar dict headers: The headers of the HTTP response.
    :ivar str body: The body of the HTTP response.
    """

    def __init__(self, response: RequestsResponse):
        """
        Initializes a Response object.

        :param RequestsResponse response: The requests.Response object.
        """
        self.status_code = response.status_code
        self.headers = response.headers
        self.body = self._get_response_body(response)

    def __str__(self) -> str:
        """
        Return a string representation of the Response object.

        :return: A string representation of the Response object.
        :rtype: str
        """
        return f"Response(status_code={self.status_code}, headers={self.headers}, body={self.body})"

    def _get_response_body(self, response: RequestsResponse) -> str:
        """
        Extracts the response body from a given HTTP response.

        This method attempts to parse the response body based on its content type.
        If the content type is JSON, it tries to parse the body as JSON.
        If the content type is text or XML, it returns the raw text.
        If the content type is 'application/x-www-form-urlencoded', it parses the body as a query string.
        For all other content types, it returns the raw binary content.

        :param RequestsResponse response: The HTTP response received from a request.
        :return: The parsed response body.
        :rtype: str or dict or bytes
        """
        try:
            return response.json()
        except ValueError:
            content_type = response.headers.get("Content-Type", "").lower()
            if "text/" in content_type or content_type == "application/xml":
                return response.text
            elif content_type == "application/x-www-form-urlencoded":
                parsed_response = parse_qs(response.text)
                return {k: v[0] for k, v in parsed_response.items()}
            else:
                return response.content



# This file was generated by liblab | https://liblab.com/

from typing import Union
from .utils.json_map import JsonMap
from .base import BaseModel
from .base import OneOfBaseModel


class AdjustmentToUpdateAmountGuard(OneOfBaseModel):
    class_list = {"str": str, "float": float}


AdjustmentToUpdateAmount = Union[str, float]


@JsonMap({})
class AdjustmentToUpdate(BaseModel):
    """Details of adjustment to update

    :param amount: Amount of adjustment., defaults to None
    :type amount: AdjustmentToUpdateAmount, optional
    :param title: Title of adjustment., defaults to None
    :type title: str, optional
    :param description: Description of adjustment., defaults to None
    :type description: str, optional
    :param file: File of adjustment., defaults to None
    :type file: str, optional
    """

    def __init__(
        self,
        amount: AdjustmentToUpdateAmount = None,
        title: str = None,
        description: str = None,
        file: str = None,
    ):
        if amount is not None:
            self.amount = AdjustmentToUpdateAmountGuard.return_one_of(amount)
        if title is not None:
            self.title = title
        if description is not None:
            self.description = description
        if file is not None:
            self.file = file



# This file was generated by liblab | https://liblab.com/

from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class CreatePublicToken(BaseModel):
    """CreatePublicToken

    :param scope: Access scopes., defaults to None
    :type scope: List[str], optional
    """

    def __init__(self, scope: List[str] = None):
        if scope is not None:
            self.scope = scope



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .identifier_value import IdentifierValue, IdentifierValueGuard
from .team_of_contract import TeamOfContract
from .client_of_contract_legal_entity_5 import ClientOfContractLegalEntity5


@JsonMap({"id_": "id"})
class ClientOfContract(BaseModel):
    """ClientOfContract

    :param id_: id_, defaults to None
    :type id_: IdentifierValue, optional
    :param full_name: full_name, defaults to None
    :type full_name: str, optional
    :param email: User's email address., defaults to None
    :type email: str, optional
    :param team: team
    :type team: TeamOfContract
    :param legal_entity: legal_entity
    :type legal_entity: ClientOfContractLegalEntity5
    """

    def __init__(
        self,
        team: TeamOfContract,
        legal_entity: ClientOfContractLegalEntity5,
        id_: IdentifierValue = None,
        full_name: str = None,
        email: str = None,
    ):
        if id_ is not None:
            self.id_ = IdentifierValueGuard.return_one_of(id_)
        if full_name is not None:
            self.full_name = full_name
        if email is not None:
            self.email = email
        self.team = self._define_object(team, TeamOfContract)
        self.legal_entity = self._define_object(
            legal_entity, ClientOfContractLegalEntity5
        )



# This file was generated by liblab | https://liblab.com/

from enum import Enum
from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class Provider(BaseModel):
    """Provider

    :param name: Name of the provider., defaults to None
    :type name: str, optional
    """

    def __init__(self, name: str = None):
        if name is not None:
            self.name = name


class OfferStatus(Enum):
    """An enumeration representing different categories.

    :cvar OFFERED: "Offered"
    :vartype OFFERED: str
    :cvar NOT_OFFERED: "Not offered"
    :vartype NOT_OFFERED: str
    """

    OFFERED = "Offered"
    NOT_OFFERED = "Not offered"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, OfferStatus._member_map_.values()))


@JsonMap({})
class EorContractBenefitsPlan(BaseModel):
    """EorContractBenefitsPlan

    :param name: Name of the plan., defaults to None
    :type name: str, optional
    """

    def __init__(self, name: str = None):
        if name is not None:
            self.name = name


@JsonMap({})
class EnrollmentDetailsPlan(BaseModel):
    """EnrollmentDetailsPlan

    :param name: Name of the plan., defaults to None
    :type name: str, optional
    """

    def __init__(self, name: str = None):
        if name is not None:
            self.name = name


@JsonMap({})
class Standard(BaseModel):
    """Standard

    :param monthly_cost: Standard monthly cost., defaults to None
    :type monthly_cost: str, optional
    :param monthly_contribution: Standard monthly contribution., defaults to None
    :type monthly_contribution: str, optional
    """

    def __init__(self, monthly_cost: str = None, monthly_contribution: str = None):
        if monthly_cost is not None:
            self.monthly_cost = monthly_cost
        if monthly_contribution is not None:
            self.monthly_contribution = monthly_contribution


@JsonMap({})
class Current(BaseModel):
    """Current

    :param monthly_cost: Current monthly cost., defaults to None
    :type monthly_cost: str, optional
    :param monthly_contribution: Current monthly contribution., defaults to None
    :type monthly_contribution: str, optional
    """

    def __init__(self, monthly_cost: str = None, monthly_contribution: str = None):
        if monthly_cost is not None:
            self.monthly_cost = monthly_cost
        if monthly_contribution is not None:
            self.monthly_contribution = monthly_contribution


@JsonMap({})
class EnrollmentDetails(BaseModel):
    """EnrollmentDetails

    :param status: Status of the enrollment., defaults to None
    :type status: str, optional
    :param plan: plan, defaults to None
    :type plan: EnrollmentDetailsPlan, optional
    :param standard: standard, defaults to None
    :type standard: Standard, optional
    :param current: current, defaults to None
    :type current: Current, optional
    """

    def __init__(
        self,
        status: str = None,
        plan: EnrollmentDetailsPlan = None,
        standard: Standard = None,
        current: Current = None,
    ):
        if status is not None:
            self.status = status
        if plan is not None:
            self.plan = self._define_object(plan, EnrollmentDetailsPlan)
        if standard is not None:
            self.standard = self._define_object(standard, Standard)
        if current is not None:
            self.current = self._define_object(current, Current)


@JsonMap({"id_": "id"})
class EorContractBenefits(BaseModel):
    """EorContractBenefits

    :param id_: Unique identifier of the benefit., defaults to None
    :type id_: str, optional
    :param name: Name of the benefit., defaults to None
    :type name: str, optional
    :param provider: provider, defaults to None
    :type provider: Provider, optional
    :param offer_status: Status of the offer., defaults to None
    :type offer_status: OfferStatus, optional
    :param plan: plan, defaults to None
    :type plan: EorContractBenefitsPlan, optional
    :param enrollment_details: enrollment_details, defaults to None
    :type enrollment_details: EnrollmentDetails, optional
    """

    def __init__(
        self,
        id_: str = None,
        name: str = None,
        provider: Provider = None,
        offer_status: OfferStatus = None,
        plan: EorContractBenefitsPlan = None,
        enrollment_details: EnrollmentDetails = None,
    ):
        if id_ is not None:
            self.id_ = id_
        if name is not None:
            self.name = name
        if provider is not None:
            self.provider = self._define_object(provider, Provider)
        if offer_status is not None:
            self.offer_status = self._enum_matching(
                offer_status, OfferStatus.list(), "offer_status"
            )
        if plan is not None:
            self.plan = self._define_object(plan, EorContractBenefitsPlan)
        if enrollment_details is not None:
            self.enrollment_details = self._define_object(
                enrollment_details, EnrollmentDetails
            )



# This file was generated by liblab | https://liblab.com/

from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models.public_token_created_container import PublicTokenCreatedContainer
from ..models.create_public_token_container import CreatePublicTokenContainer


class TokenService(BaseService):

    @cast_models
    def create_public_token(
        self, request_body: CreatePublicTokenContainer
    ) -> PublicTokenCreatedContainer:
        """Create a public token to use with embedded Deel SDK components.

        :param request_body: The request body.
        :type request_body: CreatePublicTokenContainer
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: PublicTokenCreatedContainer
        """

        Validator(CreatePublicTokenContainer).validate(request_body)

        serialized_request = (
            Serializer(f"{self.base_url}/public-tokens", self.get_default_headers())
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return PublicTokenCreatedContainer._unmap(response)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .eor_holidays_rollover_type import EorHolidaysRolloverType


@JsonMap({})
class EorEntitlementListItem(BaseModel):
    """Entitlement Item for a given year.

    :param year: year, defaults to None
    :type year: float, optional
    :param vacation_allowed: Minimum vacation allowed., defaults to None
    :type vacation_allowed: str, optional
    :param sick_leave_allowed: Maximum sick leave allowed., defaults to None
    :type sick_leave_allowed: str, optional
    :param vacation_requested: Vacation days requested by the employee., defaults to None
    :type vacation_requested: str, optional
    :param vacation_approved: Vacation days approved., defaults to None
    :type vacation_approved: str, optional
    :param vacation_used: Vacation days used by the employee., defaults to None
    :type vacation_used: str, optional
    :param vacation_total: Total number of vacation days requested, approved and used., defaults to None
    :type vacation_total: str, optional
    :param sick_leave_requested: Sick days requested by the employee., defaults to None
    :type sick_leave_requested: str, optional
    :param sick_leave_approved: Sick days approved., defaults to None
    :type sick_leave_approved: str, optional
    :param sick_leave_used: Sick days used by the employee., defaults to None
    :type sick_leave_used: str, optional
    :param sick_leave_total: Total number of sick days requested, approved and used., defaults to None
    :type sick_leave_total: str, optional
    :param other_leave_requested: Other type of time off requested by the employee., defaults to None
    :type other_leave_requested: str, optional
    :param other_leave_approved: Other type of time off days approved., defaults to None
    :type other_leave_approved: str, optional
    :param other_leave_used: Other type of time off days used by the employee., defaults to None
    :type other_leave_used: str, optional
    :param other_leave_total: Total number of other type of time off days requested, approved and used., defaults to None
    :type other_leave_total: str, optional
    :param rollover_type: rollover_type, defaults to None
    :type rollover_type: EorHolidaysRolloverType, optional
    :param max_rollover_yearly: max_rollover_yearly, defaults to None
    :type max_rollover_yearly: str, optional
    """

    def __init__(
        self,
        year: float = None,
        vacation_allowed: str = None,
        sick_leave_allowed: str = None,
        vacation_requested: str = None,
        vacation_approved: str = None,
        vacation_used: str = None,
        vacation_total: str = None,
        sick_leave_requested: str = None,
        sick_leave_approved: str = None,
        sick_leave_used: str = None,
        sick_leave_total: str = None,
        other_leave_requested: str = None,
        other_leave_approved: str = None,
        other_leave_used: str = None,
        other_leave_total: str = None,
        rollover_type: EorHolidaysRolloverType = None,
        max_rollover_yearly: str = None,
    ):
        if year is not None:
            self.year = year
        if vacation_allowed is not None:
            self.vacation_allowed = vacation_allowed
        if sick_leave_allowed is not None:
            self.sick_leave_allowed = sick_leave_allowed
        if vacation_requested is not None:
            self.vacation_requested = vacation_requested
        if vacation_approved is not None:
            self.vacation_approved = vacation_approved
        if vacation_used is not None:
            self.vacation_used = vacation_used
        if vacation_total is not None:
            self.vacation_total = vacation_total
        if sick_leave_requested is not None:
            self.sick_leave_requested = sick_leave_requested
        if sick_leave_approved is not None:
            self.sick_leave_approved = sick_leave_approved
        if sick_leave_used is not None:
            self.sick_leave_used = sick_leave_used
        if sick_leave_total is not None:
            self.sick_leave_total = sick_leave_total
        if other_leave_requested is not None:
            self.other_leave_requested = other_leave_requested
        if other_leave_approved is not None:
            self.other_leave_approved = other_leave_approved
        if other_leave_used is not None:
            self.other_leave_used = other_leave_used
        if other_leave_total is not None:
            self.other_leave_total = other_leave_total
        if rollover_type is not None:
            self.rollover_type = self._enum_matching(
                rollover_type, EorHolidaysRolloverType.list(), "rollover_type"
            )
        if max_rollover_yearly is not None:
            self.max_rollover_yearly = max_rollover_yearly



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .premium_to_add import PremiumToAdd


@JsonMap({})
class PremiumToAddContainer(BaseModel):
    """PremiumToAddContainer

    :param data: data
    :type data: PremiumToAdd
    """

    def __init__(self, data: PremiumToAdd):
        self.data = self._define_object(data, PremiumToAdd)



# This file was generated by liblab | https://liblab.com/

from typing import List
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models.invoice_adjustment_type_enum import InvoiceAdjustmentTypeEnum
from ..models.invoice_adjustment_to_update_container import (
    InvoiceAdjustmentToUpdateContainer,
)
from ..models.invoice_adjustment_to_create_container import (
    InvoiceAdjustmentToCreateContainer,
)
from ..models.invoice_adjustment_status_enum import InvoiceAdjustmentStatusEnum
from ..models.invoice_adjustment_reviews_to_create_container import (
    InvoiceAdjustmentReviewsToCreateContainer,
)
from ..models.invoice_adjustment_review_to_create_container import (
    InvoiceAdjustmentReviewToCreateContainer,
)
from ..models.invoice_adjustment_list_container import InvoiceAdjustmentListContainer
from ..models.invoice_adjustment_created_container import (
    InvoiceAdjustmentCreatedContainer,
)
from ..models.invoice_adjustment_attachment_container import (
    InvoiceAdjustmentAttachmentContainer,
)
from ..models.generic_result_updated import GenericResultUpdated
from ..models.generic_result_deleted import GenericResultDeleted
from ..models.generic_result_created import GenericResultCreated
from ..models.contract_type_enum import ContractTypeEnum


class InvoicesService(BaseService):

    @cast_models
    def get_invoice_adjustments_by_contract_id(
        self,
        contract_id: str,
        contract_types: List[ContractTypeEnum] = None,
        types: List[InvoiceAdjustmentTypeEnum] = None,
        statuses: List[InvoiceAdjustmentStatusEnum] = None,
        invoice_id: str = None,
        reporter_id: str = None,
        date_from: str = None,
        date_to: str = None,
        limit: float = None,
        offset: float = None,
    ) -> InvoiceAdjustmentListContainer:
        """Retrieve invoice line items for a given contract id.

        :param contract_id: Deel contract id.
        :type contract_id: str
        :param contract_types: types of contracts to filter, defaults to None
        :type contract_types: List[ContractTypeEnum], optional
        :param types: types of invoice adjustments to filter, defaults to None
        :type types: List[InvoiceAdjustmentTypeEnum], optional
        :param statuses: statuses of invoice adjustment to filter, defaults to None
        :type statuses: List[InvoiceAdjustmentStatusEnum], optional
        :param invoice_id: ID of an existing invoice; NOTE: technically ALL query parameters are strings or array of strings, defaults to None
        :type invoice_id: str, optional
        :param reporter_id: ID of an existing profile; NOTE: technically ALL query parameters are strings or array of strings, defaults to None
        :type reporter_id: str, optional
        :param date_from: to get invoice adjustments submitted on or after given start date (inclusive), defaults to None
        :type date_from: str, optional
        :param date_to: to get invoice adjustments submitted before given end date (excludes records submitted on this date), defaults to None
        :type date_to: str, optional
        :param limit: Return a page of results with given number of records; NOTE: technically ALL query parameters are strings or array of strings, defaults to None
        :type limit: float, optional
        :param offset: Return a page of results after given index of row; NOTE: technically ALL query parameters are strings or array of strings, defaults to None
        :type offset: float, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: InvoiceAdjustmentListContainer
        """

        Validator(str).validate(contract_id)
        Validator(ContractTypeEnum).is_array().is_optional().validate(contract_types)
        Validator(InvoiceAdjustmentTypeEnum).is_array().is_optional().validate(types)
        Validator(InvoiceAdjustmentStatusEnum).is_array().is_optional().validate(
            statuses
        )
        Validator(str).is_optional().validate(invoice_id)
        Validator(str).is_optional().validate(reporter_id)
        Validator(str).is_optional().pattern("^\d{4}-\d{2}-\d{2}$").validate(date_from)
        Validator(str).is_optional().pattern("^\d{4}-\d{2}-\d{2}$").validate(date_to)
        Validator(float).is_optional().min(1).max(99).validate(limit)
        Validator(float).is_optional().min(0).max(999999999).validate(offset)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/invoice-adjustments",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .add_query("contract_types", contract_types)
            .add_query("types", types)
            .add_query("statuses", statuses)
            .add_query("invoice_id", invoice_id)
            .add_query("reporter_id", reporter_id)
            .add_query("date_from", date_from, nullable=True)
            .add_query("date_to", date_to, nullable=True)
            .add_query("limit", limit)
            .add_query("offset", offset)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return InvoiceAdjustmentListContainer._unmap(response)

    @cast_models
    def get_invoice_adjustments(
        self,
        contract_id: str = None,
        contract_types: List[ContractTypeEnum] = None,
        types: List[InvoiceAdjustmentTypeEnum] = None,
        statuses: List[InvoiceAdjustmentStatusEnum] = None,
        invoice_id: str = None,
        reporter_id: str = None,
        date_from: str = None,
        date_to: str = None,
        limit: float = None,
        offset: float = None,
    ) -> InvoiceAdjustmentListContainer:
        """Retrieve invoice adjustments. You can filter the list by providing additional parameters e.g. contract_id, contract_type etc.

        :param contract_id: Id of an Deel contract., defaults to None
        :type contract_id: str, optional
        :param contract_types: types of contracts to filter, defaults to None
        :type contract_types: List[ContractTypeEnum], optional
        :param types: types of invoice adjustments to filter, defaults to None
        :type types: List[InvoiceAdjustmentTypeEnum], optional
        :param statuses: statuses of invoice adjustment to filter, defaults to None
        :type statuses: List[InvoiceAdjustmentStatusEnum], optional
        :param invoice_id: ID of an existing invoice; NOTE: technically ALL query parameters are strings or array of strings, defaults to None
        :type invoice_id: str, optional
        :param reporter_id: ID of an existing profile; NOTE: technically ALL query parameters are strings or array of strings, defaults to None
        :type reporter_id: str, optional
        :param date_from: to get invoice adjustments submitted on or after given start date (inclusive), defaults to None
        :type date_from: str, optional
        :param date_to: to get invoice adjustments submitted before given end date (excludes records submitted on this date), defaults to None
        :type date_to: str, optional
        :param limit: Return a page of results with given number of records; NOTE: technically ALL query parameters are strings or array of strings, defaults to None
        :type limit: float, optional
        :param offset: Return a page of results after given index of row; NOTE: technically ALL query parameters are strings or array of strings, defaults to None
        :type offset: float, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: InvoiceAdjustmentListContainer
        """

        Validator(str).is_optional().validate(contract_id)
        Validator(ContractTypeEnum).is_array().is_optional().validate(contract_types)
        Validator(InvoiceAdjustmentTypeEnum).is_array().is_optional().validate(types)
        Validator(InvoiceAdjustmentStatusEnum).is_array().is_optional().validate(
            statuses
        )
        Validator(str).is_optional().validate(invoice_id)
        Validator(str).is_optional().validate(reporter_id)
        Validator(str).is_optional().pattern("^\d{4}-\d{2}-\d{2}$").validate(date_from)
        Validator(str).is_optional().pattern("^\d{4}-\d{2}-\d{2}$").validate(date_to)
        Validator(float).is_optional().min(1).max(99).validate(limit)
        Validator(float).is_optional().min(0).max(999999999).validate(offset)

        serialized_request = (
            Serializer(
                f"{self.base_url}/invoice-adjustments", self.get_default_headers()
            )
            .add_query("contract_id", contract_id)
            .add_query("contract_types", contract_types)
            .add_query("types", types)
            .add_query("statuses", statuses)
            .add_query("invoice_id", invoice_id)
            .add_query("reporter_id", reporter_id)
            .add_query("date_from", date_from, nullable=True)
            .add_query("date_to", date_to, nullable=True)
            .add_query("limit", limit)
            .add_query("offset", offset)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return InvoiceAdjustmentListContainer._unmap(response)

    @cast_models
    def create_invoice_adjustment(
        self, request_body: InvoiceAdjustmentToCreateContainer, recurring: bool = None
    ) -> InvoiceAdjustmentCreatedContainer:
        """Create an invoice adjustment using this endpoint. For example, you can add a bonus, commission, VAT %, deduction etc. to an invoice.

        :param request_body: The request body.
        :type request_body: InvoiceAdjustmentToCreateContainer
        :param recurring: Add this invoice adjustment as recurring, defaults to None
        :type recurring: bool, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: InvoiceAdjustmentCreatedContainer
        """

        Validator(InvoiceAdjustmentToCreateContainer).validate(request_body)
        Validator(bool).is_optional().validate(recurring)

        serialized_request = (
            Serializer(
                f"{self.base_url}/invoice-adjustments", self.get_default_headers()
            )
            .add_query("recurring", recurring)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return InvoiceAdjustmentCreatedContainer._unmap(response)

    @cast_models
    def update_invoice_adjustment_by_id(
        self,
        request_body: InvoiceAdjustmentToUpdateContainer,
        invoice_adjustment_id: str,
    ) -> GenericResultUpdated:
        """Update an existing invoice adjustment. It is not possible to update VAT adjustments, we recommend you to delete the existing VAT adjust and create a new one.

        :param request_body: The request body.
        :type request_body: InvoiceAdjustmentToUpdateContainer
        :param invoice_adjustment_id: ID of an existing invoice adjustment
        :type invoice_adjustment_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultUpdated
        """

        Validator(InvoiceAdjustmentToUpdateContainer).validate(request_body)
        Validator(str).validate(invoice_adjustment_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/invoice-adjustments/{{invoice_adjustment_id}}",
                self.get_default_headers(),
            )
            .add_path("invoice_adjustment_id", invoice_adjustment_id)
            .serialize()
            .set_method("PATCH")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GenericResultUpdated._unmap(response)

    @cast_models
    def delete_invoice_adjustment_by_id(
        self, invoice_adjustment_id: str, reason: str = None
    ) -> GenericResultDeleted:
        """Delete an existing invoice adjustment.

        :param invoice_adjustment_id: ID of an existing invoice adjustment
        :type invoice_adjustment_id: str
        :param reason: Reason for deleting an existing invoice adjustment, defaults to None
        :type reason: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultDeleted
        """

        Validator(str).validate(invoice_adjustment_id)
        Validator(str).is_optional().validate(reason)

        serialized_request = (
            Serializer(
                f"{self.base_url}/invoice-adjustments/{{invoice_adjustment_id}}",
                self.get_default_headers(),
            )
            .add_path("invoice_adjustment_id", invoice_adjustment_id)
            .add_query("reason", reason)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)

        return GenericResultDeleted._unmap(response)

    @cast_models
    def create_invoice_adjustment_review(
        self,
        invoice_adjustment_id: str,
        request_body: InvoiceAdjustmentReviewToCreateContainer = None,
    ) -> GenericResultCreated:
        """Review an invoice adjustment to approve or decline it.

        :param request_body: The request body., defaults to None
        :type request_body: InvoiceAdjustmentReviewToCreateContainer, optional
        :param invoice_adjustment_id: ID of an existing invoice adjustment
        :type invoice_adjustment_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultCreated
        """

        Validator(InvoiceAdjustmentReviewToCreateContainer).is_optional().validate(
            request_body
        )
        Validator(str).validate(invoice_adjustment_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/invoice-adjustments/{{invoice_adjustment_id}}/reviews",
                self.get_default_headers(),
            )
            .add_path("invoice_adjustment_id", invoice_adjustment_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GenericResultCreated._unmap(response)

    @cast_models
    def create_invoice_adjustment_reviews(
        self, request_body: InvoiceAdjustmentReviewsToCreateContainer = None
    ) -> GenericResultCreated:
        """Review multiple invoice adjustments to approve or decline a batch.

        :param request_body: The request body., defaults to None
        :type request_body: InvoiceAdjustmentReviewsToCreateContainer, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultCreated
        """

        Validator(InvoiceAdjustmentReviewsToCreateContainer).is_optional().validate(
            request_body
        )

        serialized_request = (
            Serializer(
                f"{self.base_url}/invoice-adjustments/many/reviews",
                self.get_default_headers(),
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GenericResultCreated._unmap(response)

    @cast_models
    def get_invoice_adjustments_attachment(
        self, invoice_adjustment_id: str
    ) -> InvoiceAdjustmentAttachmentContainer:
        """Retrieve Attachment file url of specified id.

        :param invoice_adjustment_id: ID of an existing invoice adjustment
        :type invoice_adjustment_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: InvoiceAdjustmentAttachmentContainer
        """

        Validator(str).validate(invoice_adjustment_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/invoice-adjustments/{{invoice_adjustment_id}}/attachment",
                self.get_default_headers(),
            )
            .add_path("invoice_adjustment_id", invoice_adjustment_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return InvoiceAdjustmentAttachmentContainer._unmap(response)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"date_": "date"})
class PaymentBreakDown(BaseModel):
    """PaymentBreakDown

    :param date_: date_, defaults to None
    :type date_: str, optional
    :param general_ledger_account: general_ledger_account, defaults to None
    :type general_ledger_account: str, optional
    :param team: team, defaults to None
    :type team: str, optional
    :param contractor_unique_identifier: Worker's unique identifier., defaults to None
    :type contractor_unique_identifier: str, optional
    :param contractor_employee_name: Worker's name., defaults to None
    :type contractor_employee_name: str, optional
    :param contractor_email: Worker's email., defaults to None
    :type contractor_email: str, optional
    :param invoice_number: Invoice number., defaults to None
    :type invoice_number: str, optional
    :param currency: Currency code., defaults to None
    :type currency: str, optional
    :param payment_currency: payment_currency, defaults to None
    :type payment_currency: str, optional
    :param receipt_number: receipt_number, defaults to None
    :type receipt_number: str, optional
    :param work: work, defaults to None
    :type work: str, optional
    :param bonus: bonus, defaults to None
    :type bonus: str, optional
    :param expenses: expenses, defaults to None
    :type expenses: str, optional
    :param commissions: commissions, defaults to None
    :type commissions: str, optional
    :param deductions: deductions, defaults to None
    :type deductions: str, optional
    :param overtime: overtime, defaults to None
    :type overtime: str, optional
    :param pro_rata: pro_rata, defaults to None
    :type pro_rata: str, optional
    :param others: others, defaults to None
    :type others: str, optional
    :param processing_fee: processing_fee, defaults to None
    :type processing_fee: str, optional
    :param adjustment: adjustment, defaults to None
    :type adjustment: str, optional
    :param total: Total due., defaults to None
    :type total: str, optional
    :param total_payment_currency: Total in payment currency., defaults to None
    :type total_payment_currency: str, optional
    :param payment_date: Date of payment., defaults to None
    :type payment_date: str, optional
    :param frequency: frequency, defaults to None
    :type frequency: str, optional
    :param contract_country: Country of the contract., defaults to None
    :type contract_country: str, optional
    :param contract_start_date: Contract start date., defaults to None
    :type contract_start_date: str, optional
    :param approvers: approvers, defaults to None
    :type approvers: str, optional
    :param approve_date: approve_date, defaults to None
    :type approve_date: str, optional
    """

    def __init__(
        self,
        date_: str = None,
        general_ledger_account: str = None,
        team: str = None,
        contractor_unique_identifier: str = None,
        contractor_employee_name: str = None,
        contractor_email: str = None,
        invoice_number: str = None,
        currency: str = None,
        payment_currency: str = None,
        receipt_number: str = None,
        work: str = None,
        bonus: str = None,
        expenses: str = None,
        commissions: str = None,
        deductions: str = None,
        overtime: str = None,
        pro_rata: str = None,
        others: str = None,
        processing_fee: str = None,
        adjustment: str = None,
        total: str = None,
        total_payment_currency: str = None,
        payment_date: str = None,
        frequency: str = None,
        contract_country: str = None,
        contract_start_date: str = None,
        approvers: str = None,
        approve_date: str = None,
    ):
        if date_ is not None:
            self.date_ = date_
        if general_ledger_account is not None:
            self.general_ledger_account = general_ledger_account
        if team is not None:
            self.team = team
        if contractor_unique_identifier is not None:
            self.contractor_unique_identifier = contractor_unique_identifier
        if contractor_employee_name is not None:
            self.contractor_employee_name = contractor_employee_name
        if contractor_email is not None:
            self.contractor_email = contractor_email
        if invoice_number is not None:
            self.invoice_number = invoice_number
        if currency is not None:
            self.currency = currency
        if payment_currency is not None:
            self.payment_currency = payment_currency
        if receipt_number is not None:
            self.receipt_number = receipt_number
        if work is not None:
            self.work = work
        if bonus is not None:
            self.bonus = bonus
        if expenses is not None:
            self.expenses = expenses
        if commissions is not None:
            self.commissions = commissions
        if deductions is not None:
            self.deductions = deductions
        if overtime is not None:
            self.overtime = overtime
        if pro_rata is not None:
            self.pro_rata = pro_rata
        if others is not None:
            self.others = others
        if processing_fee is not None:
            self.processing_fee = processing_fee
        if adjustment is not None:
            self.adjustment = adjustment
        if total is not None:
            self.total = total
        if total_payment_currency is not None:
            self.total_payment_currency = total_payment_currency
        if payment_date is not None:
            self.payment_date = payment_date
        if frequency is not None:
            self.frequency = frequency
        if contract_country is not None:
            self.contract_country = contract_country
        if contract_start_date is not None:
            self.contract_start_date = contract_start_date
        if approvers is not None:
            self.approvers = approvers
        if approve_date is not None:
            self.approve_date = approve_date



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class StateOfCountry(BaseModel):
    """It may be called states, provinces, prefectures or regions, or none at all

    :param code: Territory code.
    :type code: str
    :param name: Territory name.
    :type name: str
    """

    def __init__(self, code: str, name: str):
        self.code = code
        self.name = name



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .team import Team
from .people_payment import PeoplePayment
from .people_client_legal_entity import PeopleClientLegalEntity


@JsonMap({"id_": "id"})
class Employment(BaseModel):
    """Employment

    :param id_: Unique identifier for the employment., defaults to None
    :type id_: str, optional
    :param name: The name of the employment., defaults to None
    :type name: str, optional
    :param team: team, defaults to None
    :type team: Team, optional
    :param email: The email associated with the employment., defaults to None
    :type email: str, optional
    :param state: The state associated with the employment., defaults to None
    :type state: str, optional
    :param country: The country associated with the employment., defaults to None
    :type country: str, optional
    :param payment: payment, defaults to None
    :type payment: PeoplePayment, optional
    :param is_ended: Indicates whether the employment has ended., defaults to None
    :type is_ended: bool, optional
    :param timezone: The timezone associated with the employment., defaults to None
    :type timezone: str, optional
    :param job_title: The job title associated with the employment., defaults to None
    :type job_title: str, optional
    :param seniority: The seniority level associated with the employment., defaults to None
    :type seniority: str, optional
    :param start_date: The start date of the employment., defaults to None
    :type start_date: str, optional
    :param work_email: The work email associated with the employment., defaults to None
    :type work_email: str, optional
    :param hiring_type: The hiring type associated with the employment., defaults to None
    :type hiring_type: str, optional
    :param hiring_status: The hiring status associated with the employment., defaults to None
    :type hiring_status: str, optional
    :param completion_date: The completion date of the employment., defaults to None
    :type completion_date: str, optional
    :param contract_status: The contract status associated with the employment., defaults to None
    :type contract_status: str, optional
    :param voluntarily_left: Indicates whether the employment was left voluntarily., defaults to None
    :type voluntarily_left: bool, optional
    :param client_legal_entity: client_legal_entity, defaults to None
    :type client_legal_entity: PeopleClientLegalEntity, optional
    :param has_eor_termination: Indicates whether the employment had an EOR termination., defaults to None
    :type has_eor_termination: bool, optional
    :param contract_is_archived: Indicates whether the employment's contract is archived., defaults to None
    :type contract_is_archived: bool, optional
    """

    def __init__(
        self,
        id_: str = None,
        name: str = None,
        team: Team = None,
        email: str = None,
        state: str = None,
        country: str = None,
        payment: PeoplePayment = None,
        is_ended: bool = None,
        timezone: str = None,
        job_title: str = None,
        seniority: str = None,
        start_date: str = None,
        work_email: str = None,
        hiring_type: str = None,
        hiring_status: str = None,
        completion_date: str = None,
        contract_status: str = None,
        voluntarily_left: bool = None,
        client_legal_entity: PeopleClientLegalEntity = None,
        has_eor_termination: bool = None,
        contract_is_archived: bool = None,
    ):
        if id_ is not None:
            self.id_ = id_
        if name is not None:
            self.name = name
        if team is not None:
            self.team = self._define_object(team, Team)
        if email is not None:
            self.email = email
        if state is not None:
            self.state = state
        if country is not None:
            self.country = country
        if payment is not None:
            self.payment = self._define_object(payment, PeoplePayment)
        if is_ended is not None:
            self.is_ended = is_ended
        if timezone is not None:
            self.timezone = timezone
        if job_title is not None:
            self.job_title = job_title
        if seniority is not None:
            self.seniority = seniority
        if start_date is not None:
            self.start_date = start_date
        if work_email is not None:
            self.work_email = work_email
        if hiring_type is not None:
            self.hiring_type = hiring_type
        if hiring_status is not None:
            self.hiring_status = hiring_status
        if completion_date is not None:
            self.completion_date = completion_date
        if contract_status is not None:
            self.contract_status = contract_status
        if voluntarily_left is not None:
            self.voluntarily_left = voluntarily_left
        if client_legal_entity is not None:
            self.client_legal_entity = self._define_object(
                client_legal_entity, PeopleClientLegalEntity
            )
        if has_eor_termination is not None:
            self.has_eor_termination = has_eor_termination
        if contract_is_archived is not None:
            self.contract_is_archived = contract_is_archived



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class MisclassificationGuarantee(BaseModel):
    """MisclassificationGuarantee

    :param name: Legal costs., defaults to None
    :type name: str, optional
    :param description: Cover your legal defense costs., defaults to None
    :type description: str, optional
    """

    def __init__(self, name: str = None, description: str = None):
        if name is not None:
            self.name = name
        if description is not None:
            self.description = description


@JsonMap({})
class Penalties(BaseModel):
    """Penalties

    :param name: Penalties., defaults to None
    :type name: str, optional
    :param description: Get indemnity against tax authority penalties, liabilities and fines., defaults to None
    :type description: str, optional
    """

    def __init__(self, name: str = None, description: str = None):
        if name is not None:
            self.name = name
        if description is not None:
            self.description = description


@JsonMap({})
class ThirdPartyLiabilities(BaseModel):
    """ThirdPartyLiabilities

    :param name: Third-party liabilities., defaults to None
    :type name: str, optional
    :param description: Get indemnity against third parties up to $10k depending on the court's decision., defaults to None
    :type description: str, optional
    """

    def __init__(self, name: str = None, description: str = None):
        if name is not None:
            self.name = name
        if description is not None:
            self.description = description


@JsonMap({"id_": "id"})
class PremiumResultAdded(BaseModel):
    """PremiumResultAdded

    :param id_: Deel Premium plan id.
    :type id_: str
    :param calculated_premium: Cost for Deel Premium.
    :type calculated_premium: float
    :param misclassification_guarantee: misclassification_guarantee
    :type misclassification_guarantee: MisclassificationGuarantee
    :param penalties: penalties
    :type penalties: Penalties
    :param third_party_liabilities: third_party_liabilities
    :type third_party_liabilities: ThirdPartyLiabilities
    """

    def __init__(
        self,
        id_: str,
        calculated_premium: float,
        misclassification_guarantee: MisclassificationGuarantee,
        penalties: Penalties,
        third_party_liabilities: ThirdPartyLiabilities,
    ):
        self.id_ = id_
        self.calculated_premium = calculated_premium
        self.misclassification_guarantee = self._define_object(
            misclassification_guarantee, MisclassificationGuarantee
        )
        self.penalties = self._define_object(penalties, Penalties)
        self.third_party_liabilities = self._define_object(
            third_party_liabilities, ThirdPartyLiabilities
        )



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"type_": "type"})
class EmploymentDetailsOfContract(BaseModel):
    """EmploymentDetailsOfContract

    :param type_: type_
    :type type_: str
    :param days_per_week: days_per_week
    :type days_per_week: float
    :param hours_per_day: hours_per_day
    :type hours_per_day: float
    :param probation_period: probation_period
    :type probation_period: float
    :param paid_vacation_days: paid_vacation_days
    :type paid_vacation_days: float
    :param country: Country code., defaults to None
    :type country: str, optional
    :param state: State code., defaults to None
    :type state: str, optional
    """

    def __init__(
        self,
        type_: str,
        days_per_week: float,
        hours_per_day: float,
        probation_period: float,
        paid_vacation_days: float,
        country: str = None,
        state: str = None,
    ):
        self.type_ = type_
        self.days_per_week = days_per_week
        self.hours_per_day = hours_per_day
        self.probation_period = probation_period
        self.paid_vacation_days = paid_vacation_days
        if country is not None:
            self.country = self._pattern_matching(country, "^[A-Z]{2}$", "country")
        if state is not None:
            self.state = state



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .legal_entity_type import LegalEntityType


@JsonMap({"id_": "id"})
class BasicLegalEntity(BaseModel):
    """BasicLegalEntity

    :param id_: Unique identifier of this resource.
    :type id_: str
    :param name: name
    :type name: str
    :param country: country, defaults to None
    :type country: str, optional
    :param entity_type: entity_type
    :type entity_type: LegalEntityType
    :param entity_subtype: entity_subtype
    :type entity_subtype: str
    """

    def __init__(
        self,
        id_: str,
        name: str,
        entity_type: LegalEntityType,
        entity_subtype: str,
        country: str = None,
    ):
        self.id_ = id_
        self.name = name
        if country is not None:
            self.country = country
        self.entity_type = self._enum_matching(
            entity_type, LegalEntityType.list(), "entity_type"
        )
        self.entity_subtype = entity_subtype



# This file was generated by liblab | https://liblab.com/

from typing import Union
from .base import OneOfBaseModel


class IdentifierValueGuard(OneOfBaseModel):
    class_list = {"str": str, "int": int}


IdentifierValue = Union[str, int]



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class PageInfo(BaseModel):
    """PageInfo

    :param cursor: cursor
    :type cursor: str
    :param total_rows: total_rows
    :type total_rows: float
    """

    def __init__(self, cursor: str, total_rows: float):
        self.cursor = cursor
        self.total_rows = total_rows



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class WorkStatementPaymentDueTypeEnum(Enum):
    """An enumeration representing different categories.

    :cvar REGULAR: "REGULAR"
    :vartype REGULAR: str
    :cvar WITHIN_MONTH: "WITHIN_MONTH"
    :vartype WITHIN_MONTH: str
    """

    REGULAR = "REGULAR"
    WITHIN_MONTH = "WITHIN_MONTH"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value, WorkStatementPaymentDueTypeEnum._member_map_.values()
            )
        )



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .output_to_create_file_ref import OutputToCreateFileRef


@JsonMap({})
class OutputToCreateFileRefContainer(BaseModel):
    """OutputToCreateFileRefContainer

    :param data: data
    :type data: OutputToCreateFileRef
    """

    def __init__(self, data: OutputToCreateFileRef):
        self.data = self._define_object(data, OutputToCreateFileRef)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .payment import Payment


@JsonMap({})
class PaymentObject(BaseModel):
    """PaymentObject

    :param rows: rows, defaults to None
    :type rows: List[Payment], optional
    :param total: total, defaults to None
    :type total: float, optional
    """

    def __init__(self, rows: List[Payment] = None, total: float = None):
        if rows is not None:
            self.rows = self._define_list(rows, Payment)
        if total is not None:
            self.total = total



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .eor_timeoffs_base_item import EorTimeoffsBaseItem


@JsonMap({})
class EorTimeoffsContainer(BaseModel):
    """EorTimeoffsContainer

    :param data: List of time offs.
    :type data: List[EorTimeoffsBaseItem]
    """

    def __init__(self, data: List[EorTimeoffsBaseItem]):
        self.data = self._define_list(data, EorTimeoffsBaseItem)



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class LegalEntityType(Enum):
    """An enumeration representing different categories.

    :cvar INDIVIDUAL: "individual"
    :vartype INDIVIDUAL: str
    :cvar COMPANY: "company"
    :vartype COMPANY: str
    """

    INDIVIDUAL = "individual"
    COMPANY = "company"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, LegalEntityType._member_map_.values()))



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"id_": "id", "type_": "type"})
class WorkerDocument(BaseModel):
    """WorkerDocument

    :param id_: Document unique ID., defaults to None
    :type id_: str, optional
    :param type_: Document type., defaults to None
    :type type_: str, optional
    :param contractor_signed_at: Date the contractor singed the document., defaults to None
    :type contractor_signed_at: str, optional
    :param client_signed_at: Date the client singed the document., defaults to None
    :type client_signed_at: str, optional
    :param label: Document name., defaults to None
    :type label: str, optional
    """

    def __init__(
        self,
        id_: str = None,
        type_: str = None,
        contractor_signed_at: str = None,
        client_signed_at: str = None,
        label: str = None,
    ):
        if id_ is not None:
            self.id_ = id_
        if type_ is not None:
            self.type_ = type_
        if contractor_signed_at is not None:
            self.contractor_signed_at = contractor_signed_at
        if client_signed_at is not None:
            self.client_signed_at = client_signed_at
        if label is not None:
            self.label = label



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"id_": "id"})
class WorkerBankAccountInfo(BaseModel):
    """WorkerBankAccountInfo

    :param full_name: Full name of the user., defaults to None
    :type full_name: str, optional
    :param phone: The user's phone number., defaults to None
    :type phone: str, optional
    :param address_line1: The primary address line., defaults to None
    :type address_line1: str, optional
    :param address_line2: The secondary address line., defaults to None
    :type address_line2: str, optional
    :param city: The city of the user's address., defaults to None
    :type city: str, optional
    :param province_state: The state or province of the user's address., defaults to None
    :type province_state: str, optional
    :param postal: The user's postal or ZIP code., defaults to None
    :type postal: str, optional
    :param bank_name: Name of the user's bank., defaults to None
    :type bank_name: str, optional
    :param country_code: The country code of the user., defaults to None
    :type country_code: str, optional
    :param bank_country_code: The country code where the bank is located., defaults to None
    :type bank_country_code: str, optional
    :param swift_bic: SWIFT/BIC code for the bank., defaults to None
    :type swift_bic: str, optional
    :param account_number: The user's bank account number., defaults to None
    :type account_number: str, optional
    :param bank_code: The bank code., defaults to None
    :type bank_code: str, optional
    :param original_name: The original name of the user., defaults to None
    :type original_name: str, optional
    :param tax_id: The user's tax identification number., defaults to None
    :type tax_id: str, optional
    :param branch_code: The branch code of the user's bank., defaults to None
    :type branch_code: str, optional
    :param currency_code: The currency code for transactions., defaults to None
    :type currency_code: str, optional
    :param bank_branch_name: The name of the user's bank branch., defaults to None
    :type bank_branch_name: str, optional
    :param iban: The International Bank Account Number (IBAN)., defaults to None
    :type iban: str, optional
    :param email: The user's email address., defaults to None
    :type email: str, optional
    :param rib_number: The RIB(Relevé d'Identité Bancaire)., defaults to None
    :type rib_number: str, optional
    :param account_type: Bank account type., defaults to None
    :type account_type: str, optional
    :param ach_routing_number: The ACH (Automated Clearing House) Routing Number., defaults to None
    :type ach_routing_number: str, optional
    :param id_: Unique identifier of this resource.
    :type id_: str
    :param created_at: Long date-time format following ISO-8601
    :type created_at: str
    :param updated_at: Long date-time format following ISO-8601
    :type updated_at: str
    :param status: Current status of bank account, defaults to None
    :type status: str, optional
    """

    def __init__(
        self,
        id_: str,
        created_at: str,
        updated_at: str,
        full_name: str = None,
        phone: str = None,
        address_line1: str = None,
        address_line2: str = None,
        city: str = None,
        province_state: str = None,
        postal: str = None,
        bank_name: str = None,
        country_code: str = None,
        bank_country_code: str = None,
        swift_bic: str = None,
        account_number: str = None,
        bank_code: str = None,
        original_name: str = None,
        tax_id: str = None,
        branch_code: str = None,
        currency_code: str = None,
        bank_branch_name: str = None,
        iban: str = None,
        email: str = None,
        rib_number: str = None,
        account_type: str = None,
        ach_routing_number: str = None,
        status: str = None,
    ):
        if full_name is not None:
            self.full_name = full_name
        if phone is not None:
            self.phone = phone
        if address_line1 is not None:
            self.address_line1 = address_line1
        if address_line2 is not None:
            self.address_line2 = address_line2
        if city is not None:
            self.city = city
        if province_state is not None:
            self.province_state = province_state
        if postal is not None:
            self.postal = postal
        if bank_name is not None:
            self.bank_name = bank_name
        if country_code is not None:
            self.country_code = country_code
        if bank_country_code is not None:
            self.bank_country_code = bank_country_code
        if swift_bic is not None:
            self.swift_bic = swift_bic
        if account_number is not None:
            self.account_number = account_number
        if bank_code is not None:
            self.bank_code = bank_code
        if original_name is not None:
            self.original_name = original_name
        if tax_id is not None:
            self.tax_id = tax_id
        if branch_code is not None:
            self.branch_code = branch_code
        if currency_code is not None:
            self.currency_code = currency_code
        if bank_branch_name is not None:
            self.bank_branch_name = bank_branch_name
        if iban is not None:
            self.iban = iban
        if email is not None:
            self.email = email
        if rib_number is not None:
            self.rib_number = rib_number
        if account_type is not None:
            self.account_type = account_type
        if ach_routing_number is not None:
            self.ach_routing_number = ach_routing_number
        self.id_ = id_
        self.created_at = created_at
        self.updated_at = updated_at
        if status is not None:
            self.status = status



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class BankAccountStatus(Enum):
    """An enumeration representing different categories.

    :cvar NEW: "new"
    :vartype NEW: str
    :cvar PENDING: "pending"
    :vartype PENDING: str
    :cvar APPROVED: "approved"
    :vartype APPROVED: str
    :cvar FAILED: "failed"
    :vartype FAILED: str
    """

    NEW = "new"
    PENDING = "pending"
    APPROVED = "approved"
    FAILED = "failed"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, BankAccountStatus._member_map_.values()))



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class HrisJobInformationTitleId(BaseModel):
    """HrisJobInformationTitleId

    :param seniority_id: seniority_id
    :type seniority_id: float
    :param job_title_id: job_title_id
    :type job_title_id: float
    """

    def __init__(self, seniority_id: float, job_title_id: float):
        self.seniority_id = seniority_id
        self.job_title_id = job_title_id



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"id_": "id", "from_": "from"})
class Payslips(BaseModel):
    """Payslips

    :param id_: Unique identifier for the payslip.
    :type id_: str
    :param from_: Start date of the payslip.
    :type from_: str
    :param to: End date of the payslip.
    :type to: str
    :param status: Status of the payslip.
    :type status: str
    """

    def __init__(self, id_: str, from_: str, to: str, status: str):
        self.id_ = id_
        self.from_ = from_
        self.to = to
        self.status = status



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class FileObject(BaseModel):
    """This is the file you will upload in a multi-part form.

    :param file: Upload the file you want to attach to this entry., defaults to None
    :type file: str, optional
    """

    def __init__(self, file: str = None):
        if file is not None:
            self.file = file



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class HrisCompensation(BaseModel):
    """HrisCompensation

    :param gross_annual_salary: gross_annual_salary
    :type gross_annual_salary: float
    :param currency: currency
    :type currency: str
    """

    def __init__(self, gross_annual_salary: float, currency: str):
        self.gross_annual_salary = gross_annual_salary
        self.currency = currency



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"id_": "id"})
class BasicTeam(BaseModel):
    """BasicTeam

    :param id_: Unique identifier of this resource.
    :type id_: str
    :param name: name
    :type name: str
    """

    def __init__(self, id_: str, name: str):
        self.id_ = id_
        self.name = name



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"id_": "id", "from_": "from"})
class EmployeePayslipsList(BaseModel):
    """EmployeePayslipsList

    :param id_: id_
    :type id_: str
    :param from_: from_
    :type from_: str
    :param to: to
    :type to: str
    :param filename: filename
    :type filename: str
    :param status: status
    :type status: str
    :param rejected_at: rejected_at
    :type rejected_at: str
    :param reject_reason: reject_reason
    :type reject_reason: str
    :param uploaded_at: uploaded_at
    :type uploaded_at: str
    :param published_at: published_at
    :type published_at: str
    :param schedule_publishing_date: schedule_publishing_date
    :type schedule_publishing_date: str
    :param created_at: created_at
    :type created_at: str
    :param updated_at: updated_at
    :type updated_at: str
    :param url: url
    :type url: str
    """

    def __init__(
        self,
        id_: str,
        from_: str,
        to: str,
        filename: str,
        status: str,
        rejected_at: str,
        reject_reason: str,
        uploaded_at: str,
        published_at: str,
        schedule_publishing_date: str,
        created_at: str,
        updated_at: str,
        url: str,
    ):
        self.id_ = id_
        self.from_ = from_
        self.to = to
        self.filename = filename
        self.status = status
        self.rejected_at = rejected_at
        self.reject_reason = reject_reason
        self.uploaded_at = uploaded_at
        self.published_at = published_at
        self.schedule_publishing_date = schedule_publishing_date
        self.created_at = created_at
        self.updated_at = updated_at
        self.url = url



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .invoice_status_enum import InvoiceStatusEnum


@JsonMap({"id_": "id"})
class DeelInvoice(BaseModel):
    """DeelInvoice

    :param id_: Unique identifier of the invoice.
    :type id_: str
    :param label: Label of the invoice.
    :type label: str
    :param status: Current status of the invoice.
    :type status: InvoiceStatusEnum
    :param currency: Currency code.
    :type currency: str
    :param total: Total invoice amount including Fee and VAT.
    :type total: str
    :param created_at: Long date-time format following ISO-8601
    :type created_at: str
    """

    def __init__(
        self,
        id_: str,
        label: str,
        status: InvoiceStatusEnum,
        currency: str,
        total: str,
        created_at: str,
    ):
        self.id_ = id_
        self.label = label
        self.status = self._enum_matching(status, InvoiceStatusEnum.list(), "status")
        self.currency = self._pattern_matching(currency, "^[A-Z]{3}$", "currency")
        self.total = total
        self.created_at = created_at



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .gp_contract_salary_status_enum import GpContractSalaryStatusEnum
from .gp_contract_salary_scale_enum import GpContractSalaryScaleEnum


@JsonMap({})
class GpEmployeeCompensationUpdated(BaseModel):
    """GpEmployeeCompensationUpdated

    :param status: The status of a salary amendment., defaults to None
    :type status: GpContractSalaryStatusEnum, optional
    :param scale: The scale to which a given salary applies., defaults to None
    :type scale: GpContractSalaryScaleEnum, optional
    :param salary: Compensation salary., defaults to None
    :type salary: float, optional
    :param effective_date: Short date in format ISO-8601 (YYYY-MM-DD). For example: 2022-12-31., defaults to None
    :type effective_date: str, optional
    """

    def __init__(
        self,
        status: GpContractSalaryStatusEnum = None,
        scale: GpContractSalaryScaleEnum = None,
        salary: float = None,
        effective_date: str = None,
    ):
        if status is not None:
            self.status = self._enum_matching(
                status, GpContractSalaryStatusEnum.list(), "status"
            )
        if scale is not None:
            self.scale = self._enum_matching(
                scale, GpContractSalaryScaleEnum.list(), "scale"
            )
        if salary is not None:
            self.salary = salary
        if effective_date is not None:
            self.effective_date = effective_date



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .contract_type_enum import ContractTypeEnum
from .contract_status_enum import ContractStatusEnum


@JsonMap({})
class EmploymentHolidays2(BaseModel):
    """EmploymentHolidays2

    :param allowance: Number of holidays., defaults to None
    :type allowance: float, optional
    :param start_date: Short date in format ISO-8601 (YYYY-MM-DD). For example 2022-12-31., defaults to None
    :type start_date: str, optional
    """

    def __init__(self, allowance: float = None, start_date: str = None):
        if allowance is not None:
            self.allowance = allowance
        if start_date is not None:
            self.start_date = self._pattern_matching(
                start_date, "^\d{4}-\d{2}-\d{2}$", "start_date"
            )


@JsonMap({"type_": "type"})
class GpContractCreatedEmployment(BaseModel):
    """GpContractCreatedEmployment

    :param start_date: Short date in format ISO-8601 (YYYY-MM-DD). For example 2022-12-31., defaults to None
    :type start_date: str, optional
    :param end_date: Short date in format ISO-8601 (YYYY-MM-DD). For example 2022-12-31., defaults to None
    :type end_date: str, optional
    :param country: Country of employment., defaults to None
    :type country: str, optional
    :param state: State code of the state/province where this person will be employed., defaults to None
    :type state: str, optional
    :param type_: Is it a full-time contract or a part-time contract?, defaults to None
    :type type_: str, optional
    :param work_visa_required: Do you require Deel to apply for a work visa for this person?, defaults to None
    :type work_visa_required: bool, optional
    :param holidays: holidays, defaults to None
    :type holidays: EmploymentHolidays2, optional
    """

    def __init__(
        self,
        start_date: str = None,
        end_date: str = None,
        country: str = None,
        state: str = None,
        type_: str = None,
        work_visa_required: bool = None,
        holidays: EmploymentHolidays2 = None,
    ):
        if start_date is not None:
            self.start_date = self._pattern_matching(
                start_date, "^\d{4}-\d{2}-\d{2}$", "start_date"
            )
        if end_date is not None:
            self.end_date = self._pattern_matching(
                end_date, "^\d{4}-\d{2}-\d{2}$", "end_date"
            )
        if country is not None:
            self.country = country
        if state is not None:
            self.state = state
        if type_ is not None:
            self.type_ = type_
        if work_visa_required is not None:
            self.work_visa_required = work_visa_required
        if holidays is not None:
            self.holidays = self._define_object(holidays, EmploymentHolidays2)


@JsonMap({})
class ClientLegalEntity3(BaseModel):
    """ClientLegalEntity3

    :param name: Legal entity name., defaults to None
    :type name: str, optional
    """

    def __init__(self, name: str = None):
        if name is not None:
            self.name = name


@JsonMap({})
class GpContractCreatedClient(BaseModel):
    """GpContractCreatedClient

    :param legal_entity: legal_entity, defaults to None
    :type legal_entity: ClientLegalEntity3, optional
    """

    def __init__(self, legal_entity: ClientLegalEntity3 = None):
        if legal_entity is not None:
            self.legal_entity = self._define_object(legal_entity, ClientLegalEntity3)


@JsonMap({})
class GpContractCreatedCompensationDetails(BaseModel):
    """GpContractCreatedCompensationDetails

    :param salary: Gross annual/month salary., defaults to None
    :type salary: float, optional
    :param currency: Currency of gross annual/month salary., defaults to None
    :type currency: str, optional
    :param scale: Scale at which the salary is paid., defaults to None
    :type scale: str, optional
    """

    def __init__(self, salary: float = None, currency: str = None, scale: str = None):
        if salary is not None:
            self.salary = salary
        if currency is not None:
            self.currency = currency
        if scale is not None:
            self.scale = scale


@JsonMap({})
class GpContractCreatedEmployee(BaseModel):
    """GpContractCreatedEmployee

    :param first_name: Employee's first name., defaults to None
    :type first_name: str, optional
    :param last_name: Employee's last name., defaults to None
    :type last_name: str, optional
    :param email: Employee's email address., defaults to None
    :type email: str, optional
    :param employee_number: Employee's number., defaults to None
    :type employee_number: str, optional
    """

    def __init__(
        self,
        first_name: str = None,
        last_name: str = None,
        email: str = None,
        employee_number: str = None,
    ):
        if first_name is not None:
            self.first_name = first_name
        if last_name is not None:
            self.last_name = last_name
        if email is not None:
            self.email = email
        if employee_number is not None:
            self.employee_number = employee_number


@JsonMap({"id_": "id", "type_": "type"})
class GpContractCreated(BaseModel):
    """GpContractCreated

    :param id_: Id of the contract created., defaults to None
    :type id_: str, optional
    :param type_: Type of a contract, defaults to None
    :type type_: ContractTypeEnum, optional
    :param created_at: Long date-time format following ISO-8601, defaults to None
    :type created_at: str, optional
    :param status: Status of a contract in Deel workflow, defaults to None
    :type status: ContractStatusEnum, optional
    :param job_title: Employee's job title., defaults to None
    :type job_title: str, optional
    :param employment: employment, defaults to None
    :type employment: GpContractCreatedEmployment, optional
    :param client: client, defaults to None
    :type client: GpContractCreatedClient, optional
    :param compensation_details: compensation_details, defaults to None
    :type compensation_details: GpContractCreatedCompensationDetails, optional
    :param employee: employee, defaults to None
    :type employee: GpContractCreatedEmployee, optional
    """

    def __init__(
        self,
        id_: str = None,
        type_: ContractTypeEnum = None,
        created_at: str = None,
        status: ContractStatusEnum = None,
        job_title: str = None,
        employment: GpContractCreatedEmployment = None,
        client: GpContractCreatedClient = None,
        compensation_details: GpContractCreatedCompensationDetails = None,
        employee: GpContractCreatedEmployee = None,
    ):
        if id_ is not None:
            self.id_ = id_
        if type_ is not None:
            self.type_ = self._enum_matching(type_, ContractTypeEnum.list(), "type_")
        if created_at is not None:
            self.created_at = created_at
        if status is not None:
            self.status = self._enum_matching(
                status, ContractStatusEnum.list(), "status"
            )
        if job_title is not None:
            self.job_title = job_title
        if employment is not None:
            self.employment = self._define_object(
                employment, GpContractCreatedEmployment
            )
        if client is not None:
            self.client = self._define_object(client, GpContractCreatedClient)
        if compensation_details is not None:
            self.compensation_details = self._define_object(
                compensation_details, GpContractCreatedCompensationDetails
            )
        if employee is not None:
            self.employee = self._define_object(employee, GpContractCreatedEmployee)



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class ContractTypeEnum(Enum):
    """An enumeration representing different categories.

    :cvar ONGOING_TIME_BASED: "ongoing_time_based"
    :vartype ONGOING_TIME_BASED: str
    :cvar MILESTONES: "milestones"
    :vartype MILESTONES: str
    :cvar TIME_BASED: "time_based"
    :vartype TIME_BASED: str
    :cvar PAY_AS_YOU_GO_TIME_BASED: "pay_as_you_go_time_based"
    :vartype PAY_AS_YOU_GO_TIME_BASED: str
    :cvar COMMISSION: "commission"
    :vartype COMMISSION: str
    :cvar PAYG_MILESTONES: "payg_milestones"
    :vartype PAYG_MILESTONES: str
    :cvar PAYG_TASKS: "payg_tasks"
    :vartype PAYG_TASKS: str
    :cvar EOR: "eor"
    :vartype EOR: str
    :cvar UNKNOWN: "unknown"
    :vartype UNKNOWN: str
    :cvar EMPLOYEE: "employee"
    :vartype EMPLOYEE: str
    :cvar GLOBAL_PAYROLL: "global_payroll"
    :vartype GLOBAL_PAYROLL: str
    :cvar SHIELD_MSA: "shield_msa"
    :vartype SHIELD_MSA: str
    :cvar HRIS_DIRECT_EMPLOYEE: "hris_direct_employee"
    :vartype HRIS_DIRECT_EMPLOYEE: str
    :cvar PEO: "peo"
    :vartype PEO: str
    """

    ONGOING_TIME_BASED = "ongoing_time_based"
    MILESTONES = "milestones"
    TIME_BASED = "time_based"
    PAY_AS_YOU_GO_TIME_BASED = "pay_as_you_go_time_based"
    COMMISSION = "commission"
    PAYG_MILESTONES = "payg_milestones"
    PAYG_TASKS = "payg_tasks"
    EOR = "eor"
    UNKNOWN = "unknown"
    EMPLOYEE = "employee"
    GLOBAL_PAYROLL = "global_payroll"
    SHIELD_MSA = "shield_msa"
    HRIS_DIRECT_EMPLOYEE = "hris_direct_employee"
    PEO = "peo"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, ContractTypeEnum._member_map_.values()))



# This file was generated by liblab | https://liblab.com/

from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models.eor_payslips_list_container import EorPayslipsListContainer
from ..models.eor_payslip_download_url_container import EorPayslipDownloadUrlContainer
from ..models.eor_employee_cost_calculation_response_container import (
    EorEmployeeCostCalculationResponseContainer,
)
from ..models.eor_employee_cost_calculation_request_body_container import (
    EorEmployeeCostCalculationRequestBodyContainer,
)
from ..models.eor_country_validations_container import EorCountryValidationsContainer
from ..models.eor_contract_to_create_container import EorContractToCreateContainer
from ..models.eor_contract_created_container import EorContractCreatedContainer
from ..models.eor_contract_benefits_container import EorContractBenefitsContainer


class EorService(BaseService):

    @cast_models
    def get_eor_country_validations(
        self, country_code: str
    ) -> EorCountryValidationsContainer:
        """Retrieve the hiring guide data for a country. This data can be used to create Employee of Record (EOR) contract quotes.

        :param country_code: Country code.
        :type country_code: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: EorCountryValidationsContainer
        """

        Validator(str).min_length(2).max_length(2).pattern("^[A-Z]{2}$").validate(
            country_code
        )

        serialized_request = (
            Serializer(
                f"{self.base_url}/eor/validations/{{country_code}}",
                self.get_default_headers(),
            )
            .add_path("country_code", country_code)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return EorCountryValidationsContainer._unmap(response)

    @cast_models
    def get_eor_worker_payslips(self, worker_id: str) -> EorPayslipsListContainer:
        """Get of payslips for an employee.

        :param worker_id: Unique identifier for a worker.
        :type worker_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: EorPayslipsListContainer
        """

        Validator(str).validate(worker_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/eor/workers/{{worker_id}}/payslips",
                self.get_default_headers(),
            )
            .add_path("worker_id", worker_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return EorPayslipsListContainer._unmap(response)

    @cast_models
    def get_eor_worker_payslip_download_url(
        self, worker_id: str, payslip_id: str
    ) -> EorPayslipDownloadUrlContainer:
        """Get download url for EOR payslip.

        :param worker_id: Unique identifier for a worker.
        :type worker_id: str
        :param payslip_id: Unique identifier for a payslip.
        :type payslip_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: EorPayslipDownloadUrlContainer
        """

        Validator(str).validate(worker_id)
        Validator(str).validate(payslip_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/eor/workers/{{worker_id}}/payslips/{{payslip_id}}/download",
                self.get_default_headers(),
            )
            .add_path("worker_id", worker_id)
            .add_path("payslip_id", payslip_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return EorPayslipDownloadUrlContainer._unmap(response)

    @cast_models
    def calculate_eor_employment_cost(
        self, request_body: EorEmployeeCostCalculationRequestBodyContainer = None
    ) -> EorEmployeeCostCalculationResponseContainer:
        """Determine EOR employee costs across the globe.

        :param request_body: The request body., defaults to None
        :type request_body: EorEmployeeCostCalculationRequestBodyContainer, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: EorEmployeeCostCalculationResponseContainer
        """

        Validator(
            EorEmployeeCostCalculationRequestBodyContainer
        ).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url}/eor/employment_cost", self.get_default_headers()
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return EorEmployeeCostCalculationResponseContainer._unmap(response)

    @cast_models
    def get_eor_contract_benefits(
        self, contract_id: str
    ) -> EorContractBenefitsContainer:
        """Retrieve EOR contract benefits

        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: EorContractBenefitsContainer
        """

        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/eor/{{contract_id}}/benefits",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return EorContractBenefitsContainer._unmap(response)

    @cast_models
    def create_eor_contract(
        self, request_body: EorContractToCreateContainer
    ) -> EorContractCreatedContainer:
        """Create an Employee of Record (EOR) contract quote. The endpoints creates a contract quote request. Deel will process the information and get back with a quote for this contract.

        :param request_body: The request body.
        :type request_body: EorContractToCreateContainer
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: EorContractCreatedContainer
        """

        Validator(EorContractToCreateContainer).validate(request_body)

        serialized_request = (
            Serializer(f"{self.base_url}/eor", self.get_default_headers())
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return EorContractCreatedContainer._unmap(response)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .eor_timeoffs_employee_item import EorTimeoffsEmployeeItem


@JsonMap({})
class EorClientTimeoffs(BaseModel):
    """EorClientTimeoffs

    :param employees: List of employee time offs.
    :type employees: List[EorTimeoffsEmployeeItem]
    :param total_requests: total_requests
    :type total_requests: float
    """

    def __init__(self, employees: List[EorTimeoffsEmployeeItem], total_requests: float):
        self.employees = self._define_list(employees, EorTimeoffsEmployeeItem)
        self.total_requests = total_requests



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"id_": "id"})
class MilestoneReviewedBy(BaseModel):
    """Reviewer's information.

    :param id_: Unique identifier of this resource.
    :type id_: str
    :param full_name: Reviewer's full name.
    :type full_name: str
    """

    def __init__(self, id_: str, full_name: str):
        self.id_ = id_
        self.full_name = full_name


@JsonMap({"id_": "id"})
class MilestoneReportedBy(BaseModel):
    """Milestone creator.

    :param id_: Unique identifier of this resource.
    :type id_: str
    :param full_name: Reporter's full name.
    :type full_name: str
    """

    def __init__(self, id_: str, full_name: str):
        self.id_ = id_
        self.full_name = full_name


@JsonMap({"id_": "id"})
class Milestone(BaseModel):
    """Milestone

    :param id_: Unique identifier of this resource.
    :type id_: str
    :param amount: Amount to be paid for the milestone.
    :type amount: str
    :param approval_requested: Indicates if the contractor has requested the approval for the milestone.
    :type approval_requested: bool
    :param reviewed_by: Reviewer's information.
    :type reviewed_by: MilestoneReviewedBy
    :param reported_by: Milestone creator.
    :type reported_by: MilestoneReportedBy
    :param title: Title of milestone.
    :type title: str
    :param description: Description of milestone.
    :type description: str
    :param status: Current status of milestone.
    :type status: str
    :param created_at: Long date-time format following ISO-8601
    :type created_at: str
    """

    def __init__(
        self,
        id_: str,
        amount: str,
        approval_requested: bool,
        reviewed_by: MilestoneReviewedBy,
        reported_by: MilestoneReportedBy,
        title: str,
        description: str,
        status: str,
        created_at: str,
    ):
        self.id_ = id_
        self.amount = amount
        self.approval_requested = approval_requested
        self.reviewed_by = self._define_object(reviewed_by, MilestoneReviewedBy)
        self.reported_by = self._define_object(reported_by, MilestoneReportedBy)
        self.title = title
        self.description = description
        self.status = status
        self.created_at = created_at



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .employee_contract_signature_to_create import EmployeeContractSignatureToCreate


@JsonMap({})
class EmployeeContractSignatureToCreateContainer(BaseModel):
    """EmployeeContractSignatureToCreateContainer

    :param data: data
    :type data: EmployeeContractSignatureToCreate
    """

    def __init__(self, data: EmployeeContractSignatureToCreate):
        self.data = self._define_object(data, EmployeeContractSignatureToCreate)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .identifier_value import IdentifierValue, IdentifierValueGuard


@JsonMap({"id_": "id"})
class ContractTemplate(BaseModel):
    """Contract template details.

    :param id_: id_, defaults to None
    :type id_: IdentifierValue, optional
    :param title: Title of a contract template, defaults to None
    :type title: str, optional
    """

    def __init__(self, id_: IdentifierValue = None, title: str = None):
        if id_ is not None:
            self.id_ = IdentifierValueGuard.return_one_of(id_)
        if title is not None:
            self.title = title



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .update_worker_working_location import UpdateWorkerWorkingLocation


@JsonMap({})
class UpdateWorkerWorkingLocationContainer(BaseModel):
    """UpdateWorkerWorkingLocationContainer

    :param data: data
    :type data: UpdateWorkerWorkingLocation
    """

    def __init__(self, data: UpdateWorkerWorkingLocation):
        self.data = self._define_object(data, UpdateWorkerWorkingLocation)



# This file was generated by liblab | https://liblab.com/

from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models.create_admin_user_response_container import (
    CreateAdminUserResponseContainer,
)
from ..models.admin_users_container import AdminUsersContainer
from ..models.admin_user_create_container import AdminUserCreateContainer


class ManagersService(BaseService):

    @cast_models
    def get_managers(
        self, limit: str = None, offset: str = None
    ) -> AdminUsersContainer:
        """List all organization managers.

        :param limit: Maximum number of records to return., defaults to None
        :type limit: str, optional
        :param offset: Offset/index of record for the next page of records to return., defaults to None
        :type offset: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Basic information of the new admin user.
        :rtype: AdminUsersContainer
        """

        Validator(str).is_optional().validate(limit)
        Validator(str).is_optional().validate(offset)

        serialized_request = (
            Serializer(f"{self.base_url}/managers", self.get_default_headers())
            .add_query("limit", limit)
            .add_query("offset", offset)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return AdminUsersContainer._unmap(response)

    @cast_models
    def create_manager(
        self, request_body: AdminUserCreateContainer
    ) -> CreateAdminUserResponseContainer:
        """Create a new organization manager.

        :param request_body: The request body.
        :type request_body: AdminUserCreateContainer
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Basic information of the new admin user.
        :rtype: CreateAdminUserResponseContainer
        """

        Validator(AdminUserCreateContainer).validate(request_body)

        serialized_request = (
            Serializer(f"{self.base_url}/managers", self.get_default_headers())
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return CreateAdminUserResponseContainer._unmap(response)



# This file was generated by liblab | https://liblab.com/

from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.working_locations_container import WorkingLocationsContainer
from ..models.utils.cast_models import cast_models
from ..models.team_list_container import TeamListContainer
from ..models.organization_list_container import OrganizationListContainer
from ..models.legal_entity_list_container import LegalEntityListContainer
from ..models.departments_container import DepartmentsContainer
from ..models.agreement_list_container import AgreementListContainer


class OrganizationsService(BaseService):

    @cast_models
    def get_legal_entity_list(self, type_: str = None) -> LegalEntityListContainer:
        """Retrieve a list of legal entities in your account.

        :param type_: Return list of global payroll legal entities if provided, defaults to None
        :type type_: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: LegalEntityListContainer
        """

        Validator(str).is_optional().validate(type_)

        serialized_request = (
            Serializer(f"{self.base_url}/legal-entities", self.get_default_headers())
            .add_query("type", type_)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return LegalEntityListContainer._unmap(response)

    @cast_models
    def get_organizations(self) -> OrganizationListContainer:
        """Retrieve the current organization details. Organization is automatically detected from the auth token.

        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: OrganizationListContainer
        """

        serialized_request = (
            Serializer(f"{self.base_url}/organizations", self.get_default_headers())
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return OrganizationListContainer._unmap(response)

    @cast_models
    def get_teams(self) -> TeamListContainer:
        """Retrieve a list of teams in your organization.

        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: TeamListContainer
        """

        serialized_request = (
            Serializer(f"{self.base_url}/teams", self.get_default_headers())
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return TeamListContainer._unmap(response)

    @cast_models
    def get_agreements(
        self, contract_id: str = None, limit: str = None, offset: str = None
    ) -> AgreementListContainer:
        """This end-point returns a list of your agreements with Deel.

        :param contract_id: Deel contract id., defaults to None
        :type contract_id: str, optional
        :param limit: Maximum number of records to return., defaults to None
        :type limit: str, optional
        :param offset: Offset/index of record for the next page of records to return., defaults to None
        :type offset: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: AgreementListContainer
        """

        Validator(str).is_optional().validate(contract_id)
        Validator(str).is_optional().validate(limit)
        Validator(str).is_optional().validate(offset)

        serialized_request = (
            Serializer(f"{self.base_url}/agreements", self.get_default_headers())
            .add_query("contract_id", contract_id)
            .add_query("limit", limit)
            .add_query("offset", offset)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return AgreementListContainer._unmap(response)

    @cast_models
    def get_departments(self) -> DepartmentsContainer:
        """Get list of organization departments.

        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: DepartmentsContainer
        """

        serialized_request = (
            Serializer(f"{self.base_url}/departments", self.get_default_headers())
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return DepartmentsContainer._unmap(response)

    @cast_models
    def get_working_locations(self) -> WorkingLocationsContainer:
        """Get organization working locations.

        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: WorkingLocationsContainer
        """

        serialized_request = (
            Serializer(f"{self.base_url}/working-locations", self.get_default_headers())
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return WorkingLocationsContainer._unmap(response)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .eor_entitlements import EorEntitlements


@JsonMap({})
class EorEntitlementsContainer(BaseModel):
    """EorEntitlementsContainer

    :param data: Time-off Entitlements for EOR contracts.
    :type data: EorEntitlements
    """

    def __init__(self, data: EorEntitlements):
        self.data = self._define_object(data, EorEntitlements)



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class GpContractSalaryStatusEnum(Enum):
    """An enumeration representing different categories.

    :cvar ACTIVE: "ACTIVE"
    :vartype ACTIVE: str
    :cvar UPCOMING: "UPCOMING"
    :vartype UPCOMING: str
    :cvar OUTDATED: "OUTDATED"
    :vartype OUTDATED: str
    """

    ACTIVE = "ACTIVE"
    UPCOMING = "UPCOMING"
    OUTDATED = "OUTDATED"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, GpContractSalaryStatusEnum._member_map_.values())
        )



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class ContractCustomField(BaseModel):
    """Customized attributes on contracts (Employee ID, Project code, etc).

    :param name: Custom field property name., defaults to None
    :type name: str, optional
    :param value: Custom field property value., defaults to None
    :type value: str, optional
    """

    def __init__(self, name: str = None, value: str = None):
        if name is not None:
            self.name = name
        if value is not None:
            self.value = value



# This file was generated by liblab | https://liblab.com/

from enum import Enum
from .utils.json_map import JsonMap
from .base import BaseModel


class InvoiceAdjustmentReviewToCreateStatus(Enum):
    """An enumeration representing different categories.

    :cvar APPROVED: "approved"
    :vartype APPROVED: str
    :cvar DECLINED: "declined"
    :vartype DECLINED: str
    """

    APPROVED = "approved"
    DECLINED = "declined"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                InvoiceAdjustmentReviewToCreateStatus._member_map_.values(),
            )
        )


@JsonMap({})
class InvoiceAdjustmentReviewToCreate(BaseModel):
    """InvoiceAdjustmentReviewToCreate

    :param status: status
    :type status: InvoiceAdjustmentReviewToCreateStatus
    :param reason: reason
    :type reason: str
    """

    def __init__(self, status: InvoiceAdjustmentReviewToCreateStatus, reason: str):
        self.status = self._enum_matching(
            status, InvoiceAdjustmentReviewToCreateStatus.list(), "status"
        )
        self.reason = reason



# This file was generated by liblab | https://liblab.com/

from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models.off_cycle_payment_to_create_container import (
    OffCyclePaymentToCreateContainer,
)
from ..models.off_cycle_payment_list_container import OffCyclePaymentListContainer
from ..models.off_cycle_payment_container import OffCyclePaymentContainer
from ..models.generic_result_created_with_id import GenericResultCreatedWithId


class OffCyclePaymentsService(BaseService):

    @cast_models
    def get_off_cycle_payments_by_contract(
        self, contract_id: str
    ) -> OffCyclePaymentListContainer:
        """Retrieve a list of off-cycle payments for the given contract id.

        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: OffCyclePaymentListContainer
        """

        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/off-cycle-payments",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return OffCyclePaymentListContainer._unmap(response)

    @cast_models
    def create_off_cycle_payment(
        self, contract_id: str, request_body: OffCyclePaymentToCreateContainer = None
    ) -> GenericResultCreatedWithId:
        """Add a new invoice line-item for the purpose of off-cycle payment for a given contract.

        :param request_body: The request body., defaults to None
        :type request_body: OffCyclePaymentToCreateContainer, optional
        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultCreatedWithId
        """

        Validator(OffCyclePaymentToCreateContainer).is_optional().validate(request_body)
        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/off-cycle-payments",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GenericResultCreatedWithId._unmap(response)

    @cast_models
    def get_off_cycle_payment_by_contract_and_id(
        self, contract_id: str, offcycle_payment_id: str
    ) -> OffCyclePaymentContainer:
        """Retrieve a single off-cycle payment.

        :param contract_id: Deel contract id.
        :type contract_id: str
        :param offcycle_payment_id: ID of off-cycle payment to return
        :type offcycle_payment_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: OffCyclePaymentContainer
        """

        Validator(str).validate(contract_id)
        Validator(str).validate(offcycle_payment_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/off-cycle-payments/{{offcycle_payment_id}}",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .add_path("offcycle_payment_id", offcycle_payment_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return OffCyclePaymentContainer._unmap(response)



# This file was generated by liblab | https://liblab.com/

from enum import Enum
from .utils.json_map import JsonMap
from .base import BaseModel


class CreateTimeoffType(Enum):
    """An enumeration representing different categories.

    :cvar VACATION: "VACATION"
    :vartype VACATION: str
    :cvar SICK_LEAVE: "SICK_LEAVE"
    :vartype SICK_LEAVE: str
    :cvar OTHER: "OTHER"
    :vartype OTHER: str
    """

    VACATION = "VACATION"
    SICK_LEAVE = "SICK_LEAVE"
    OTHER = "OTHER"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, CreateTimeoffType._member_map_.values()))


@JsonMap({"type_": "type"})
class CreateTimeoff(BaseModel):
    """CreateTimeoff

    :param type_: Time off type.
    :type type_: CreateTimeoffType
    :param start_date: Start date of employee's time off., defaults to None
    :type start_date: str, optional
    :param end_date: End date of employee's time off., defaults to None
    :type end_date: str, optional
    :param with_multiple_dates: If true, then multiple time off days were requested. You must supply a start and end date. If false, end date is ignored
    :type with_multiple_dates: bool
    :param reason: Description for employee's time off., defaults to None
    :type reason: str, optional
    :param is_start_date_half_day: Is the start day a half day?, defaults to None
    :type is_start_date_half_day: bool, optional
    :param is_end_date_half_day: Is the start day a half day?, defaults to None
    :type is_end_date_half_day: bool, optional
    :param other_timeoff_name: Custom name for this time off., defaults to None
    :type other_timeoff_name: str, optional
    """

    def __init__(
        self,
        type_: CreateTimeoffType,
        with_multiple_dates: bool,
        start_date: str = None,
        end_date: str = None,
        reason: str = None,
        is_start_date_half_day: bool = None,
        is_end_date_half_day: bool = None,
        other_timeoff_name: str = None,
    ):
        self.type_ = self._enum_matching(type_, CreateTimeoffType.list(), "type_")
        if start_date is not None:
            self.start_date = start_date
        if end_date is not None:
            self.end_date = end_date
        self.with_multiple_dates = with_multiple_dates
        if reason is not None:
            self.reason = reason
        if is_start_date_half_day is not None:
            self.is_start_date_half_day = is_start_date_half_day
        if is_end_date_half_day is not None:
            self.is_end_date_half_day = is_end_date_half_day
        if other_timeoff_name is not None:
            self.other_timeoff_name = other_timeoff_name



# This file was generated by liblab | https://liblab.com/

from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models.update_worker_working_location_container import (
    UpdateWorkerWorkingLocationContainer,
)
from ..models.update_worker_department_container import UpdateWorkerDepartmentContainer
from ..models.timeoff_to_review_internal_container import (
    TimeoffToReviewInternalContainer,
)
from ..models.sort_dir_enum import SortDirEnum
from ..models.people_time_off_container import PeopleTimeOffContainer
from ..models.people_sort_by_enum import PeopleSortByEnum
from ..models.people_me import PeopleMe
from ..models.people_container import PeopleContainer
from ..models.people_by_id_container import PeopleByIdContainer
from ..models.internal_people_container import InternalPeopleContainer
from ..models.hris_direct_employee_container import HrisDirectEmployeeContainer
from ..models.hris_direct_employee import HrisDirectEmployee
from ..models.hiring_status_enum import HiringStatusEnum
from ..models.generic_result_updated import GenericResultUpdated
from ..models.generic_result_deleted import GenericResultDeleted
from ..models.employee_timeoffs_policies_container import (
    EmployeeTimeoffsPoliciesContainer,
)
from ..models.employee_timeoffs_entitlements_container import (
    EmployeeTimeoffsEntitlementsContainer,
)
from ..models.employee_timeoffs_creation_container import (
    EmployeeTimeoffsCreationContainer,
)
from ..models.employee_timeoffs_container import EmployeeTimeoffsContainer
from ..models.create_people_timeoff import CreatePeopleTimeoff


class PeopleService(BaseService):

    @cast_models
    def create_direct_employee(
        self, request_body: HrisDirectEmployee
    ) -> HrisDirectEmployeeContainer:
        """Create a new Hris direct employee.

        :param request_body: The request body.
        :type request_body: HrisDirectEmployee
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Hris direct employee object that was created or edited
        :rtype: HrisDirectEmployeeContainer
        """

        Validator(HrisDirectEmployee).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url}/hris/direct-employees", self.get_default_headers()
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return HrisDirectEmployeeContainer._unmap(response)

    @cast_models
    def get_internal_people_list(
        self, offset: float = None, limit: float = None
    ) -> InternalPeopleContainer:
        """Retrieve a list of internal people in your organization.

        :param offset: Return a page of results with given number of records., defaults to None
        :type offset: float, optional
        :param limit: Return a page of results with given number of records., defaults to None
        :type limit: float, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: InternalPeopleContainer
        """

        Validator(float).is_optional().min(0).max(999999999).validate(offset)
        Validator(float).is_optional().min(1).max(200).validate(limit)

        serialized_request = (
            Serializer(f"{self.base_url}/internal/people", self.get_default_headers())
            .add_query("offset", offset)
            .add_query("limit", limit)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return InternalPeopleContainer._unmap(response)

    @cast_models
    def get_people_list(
        self,
        offset: float = None,
        limit: float = None,
        search: str = None,
        sort_by: PeopleSortByEnum = None,
        sort_order: SortDirEnum = None,
        hiring_statuses: HiringStatusEnum = None,
    ) -> PeopleContainer:
        """Retrieve a list of People in your organization.

        :param offset: Return a page of results with given number of records., defaults to None
        :type offset: float, optional
        :param limit: Return a page of results with given number of records., defaults to None
        :type limit: float, optional
        :param search: Include a contract if by name or contractor name contains given search term., defaults to None
        :type search: str, optional
        :param sort_by: Sort people by given field name., defaults to None
        :type sort_by: PeopleSortByEnum, optional
        :param sort_order: Order direction of results; ascending or descending., defaults to None
        :type sort_order: SortDirEnum, optional
        :param hiring_statuses: Employee's current hiring status., defaults to None
        :type hiring_statuses: HiringStatusEnum, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: PeopleContainer
        """

        Validator(float).is_optional().min(0).max(999999999).validate(offset)
        Validator(float).is_optional().min(1).max(200).validate(limit)
        Validator(str).is_optional().validate(search)
        Validator(PeopleSortByEnum).is_optional().validate(sort_by)
        Validator(SortDirEnum).is_optional().validate(sort_order)
        Validator(HiringStatusEnum).is_optional().validate(hiring_statuses)

        serialized_request = (
            Serializer(f"{self.base_url}/people", self.get_default_headers())
            .add_query("offset", offset)
            .add_query("limit", limit)
            .add_query("search", search)
            .add_query("sort_by", sort_by)
            .add_query("sort_order", sort_order)
            .add_query("hiring_statuses[]", hiring_statuses)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return PeopleContainer._unmap(response)

    @cast_models
    def get_people_by_id(self, worker_id: str) -> PeopleByIdContainer:
        """Retrieve a single person in your organization.

        :param worker_id: Unique identifier for a worker.
        :type worker_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: PeopleByIdContainer
        """

        Validator(str).validate(worker_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/people/{{worker_id}}", self.get_default_headers()
            )
            .add_path("worker_id", worker_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return PeopleByIdContainer._unmap(response)

    @cast_models
    def update_people_department(
        self, worker_id: str, request_body: UpdateWorkerDepartmentContainer = None
    ) -> GenericResultUpdated:
        """Update worker department.

        :param request_body: The request body., defaults to None
        :type request_body: UpdateWorkerDepartmentContainer, optional
        :param worker_id: Unique identifier for a worker.
        :type worker_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultUpdated
        """

        Validator(UpdateWorkerDepartmentContainer).is_optional().validate(request_body)
        Validator(str).validate(worker_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/people/{{worker_id}}/department",
                self.get_default_headers(),
            )
            .add_path("worker_id", worker_id)
            .serialize()
            .set_method("PUT")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GenericResultUpdated._unmap(response)

    @cast_models
    def update_people_working_location(
        self, worker_id: str, request_body: UpdateWorkerWorkingLocationContainer = None
    ) -> GenericResultUpdated:
        """Update worker working location.

        :param request_body: The request body., defaults to None
        :type request_body: UpdateWorkerWorkingLocationContainer, optional
        :param worker_id: Unique identifier for a worker.
        :type worker_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultUpdated
        """

        Validator(UpdateWorkerWorkingLocationContainer).is_optional().validate(
            request_body
        )
        Validator(str).validate(worker_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/people/{{worker_id}}/working-location",
                self.get_default_headers(),
            )
            .add_path("worker_id", worker_id)
            .serialize()
            .set_method("PUT")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GenericResultUpdated._unmap(response)

    @cast_models
    def get_people(self) -> PeopleMe:
        """Retrieve the current user's profile.

        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: PeopleMe
        """

        serialized_request = (
            Serializer(f"{self.base_url}/people/me", self.get_default_headers())
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return PeopleMe._unmap(response)

    @cast_models
    def get_time_offs_for_employee(self, worker_id: str) -> EmployeeTimeoffsContainer:
        """List of time offs by worker id. Worker id can be retreived using /people endpoint.

        :param worker_id: Unique identifier for a worker.
        :type worker_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: successful operation
        :rtype: EmployeeTimeoffsContainer
        """

        Validator(str).validate(worker_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/people/{{worker_id}}/time-offs",
                self.get_default_headers(),
            )
            .add_path("worker_id", worker_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return EmployeeTimeoffsContainer._unmap(response)

    @cast_models
    def create_time_offs_for_employee(
        self, worker_id: str, request_body: dict = None
    ) -> EmployeeTimeoffsCreationContainer:
        """Add a time off request for a worker. New requests are auto-approved.

        :param request_body: The request body., defaults to None
        :type request_body: dict, optional
        :param worker_id: Unique identifier for a worker.
        :type worker_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: successful operation
        :rtype: EmployeeTimeoffsCreationContainer
        """

        Validator(dict).is_optional().validate(request_body)
        Validator(str).validate(worker_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/people/{{worker_id}}/time-offs",
                self.get_default_headers(),
            )
            .add_path("worker_id", worker_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body, "multipart/form-data")
        )

        response = self.send_request(serialized_request)

        return EmployeeTimeoffsCreationContainer._unmap(response)

    @cast_models
    def get_time_offs_entitlements_for_employee(
        self, worker_id: str
    ) -> EmployeeTimeoffsEntitlementsContainer:
        """Retrieve a list of time off entitlements for a worker.

        :param worker_id: Unique identifier for a worker.
        :type worker_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: successful operation
        :rtype: EmployeeTimeoffsEntitlementsContainer
        """

        Validator(str).validate(worker_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/people/{{worker_id}}/time-offs/entitlements",
                self.get_default_headers(),
            )
            .add_path("worker_id", worker_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return EmployeeTimeoffsEntitlementsContainer._unmap(response)

    @cast_models
    def update_time_offs_for_employee(
        self, timeoff_id: str, worker_id: str, request_body: dict = None
    ) -> EmployeeTimeoffsCreationContainer:
        """Edit a time off request for a worker.

        :param request_body: The request body., defaults to None
        :type request_body: dict, optional
        :param timeoff_id: Unique identifier for a time-off in Deel.
        :type timeoff_id: str
        :param worker_id: Unique identifier for a worker.
        :type worker_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: successful operation
        :rtype: EmployeeTimeoffsCreationContainer
        """

        Validator(dict).is_optional().validate(request_body)
        Validator(str).validate(timeoff_id)
        Validator(str).validate(worker_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/people/{{worker_id}}/time-offs/{{timeoff_id}}",
                self.get_default_headers(),
            )
            .add_path("timeoff_id", timeoff_id)
            .add_path("worker_id", worker_id)
            .serialize()
            .set_method("PATCH")
            .set_body(request_body, "multipart/form-data")
        )

        response = self.send_request(serialized_request)

        return EmployeeTimeoffsCreationContainer._unmap(response)

    @cast_models
    def delete_time_offs_for_employee(
        self, timeoff_id: str, worker_id: str
    ) -> GenericResultDeleted:
        """Delete a time off request.

        :param timeoff_id: Unique identifier for a time-off in Deel.
        :type timeoff_id: str
        :param worker_id: Unique identifier for a worker.
        :type worker_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultDeleted
        """

        Validator(str).validate(timeoff_id)
        Validator(str).validate(worker_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/people/{{worker_id}}/time-offs/{{timeoff_id}}",
                self.get_default_headers(),
            )
            .add_path("timeoff_id", timeoff_id)
            .add_path("worker_id", worker_id)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)

        return GenericResultDeleted._unmap(response)

    @cast_models
    def review_time_offs_for_employee(
        self,
        request_body: TimeoffToReviewInternalContainer,
        timeoff_id: str,
        worker_id: str,
    ) -> EmployeeTimeoffsContainer:
        """Approve or decline a time off request. New requests are auto-approved. Hence they don't need to be manually approved.

        :param request_body: The request body.
        :type request_body: TimeoffToReviewInternalContainer
        :param timeoff_id: Unique identifier for a time-off in Deel.
        :type timeoff_id: str
        :param worker_id: Unique identifier for a worker.
        :type worker_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: successful operation
        :rtype: EmployeeTimeoffsContainer
        """

        Validator(TimeoffToReviewInternalContainer).validate(request_body)
        Validator(str).validate(timeoff_id)
        Validator(str).validate(worker_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/people/{{worker_id}}/time-offs/{{timeoff_id}}/review",
                self.get_default_headers(),
            )
            .add_path("timeoff_id", timeoff_id)
            .add_path("worker_id", worker_id)
            .serialize()
            .set_method("PATCH")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return EmployeeTimeoffsContainer._unmap(response)

    @cast_models
    def get_time_offs_policies_for_employee(
        self, worker_id: str
    ) -> EmployeeTimeoffsPoliciesContainer:
        """Retrieve a list of time off policies for a worker.

        :param worker_id: Unique identifier for a worker.
        :type worker_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: successful operation
        :rtype: EmployeeTimeoffsPoliciesContainer
        """

        Validator(str).validate(worker_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/people/{{worker_id}}/time-offs/policies",
                self.get_default_headers(),
            )
            .add_path("worker_id", worker_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return EmployeeTimeoffsPoliciesContainer._unmap(response)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class MetaDataOfContractToCreate(BaseModel):
    """Additional custom info about a contract

    :param documents_required: Require the contractor to upload necessary compliance documents as per their country’s labor laws.
    :type documents_required: bool
    :param is_main_income: is_main_income, defaults to None
    :type is_main_income: bool, optional
    """

    def __init__(self, documents_required: bool, is_main_income: bool = None):
        self.documents_required = documents_required
        if is_main_income is not None:
            self.is_main_income = is_main_income



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .basic_timesheet import BasicTimesheet
from .page_info_without_cursor import PageInfoWithoutCursor


@JsonMap({})
class TimesheetListContainer(BaseModel):
    """TimesheetListContainer

    :param data: data
    :type data: List[BasicTimesheet]
    :param page: page
    :type page: PageInfoWithoutCursor
    """

    def __init__(self, data: List[BasicTimesheet], page: PageInfoWithoutCursor):
        self.data = self._define_list(data, BasicTimesheet)
        self.page = self._define_object(page, PageInfoWithoutCursor)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .eor_employee_cost_calculation_request_body import (
    EorEmployeeCostCalculationRequestBody,
)


@JsonMap({})
class EorEmployeeCostCalculationRequestBodyContainer(BaseModel):
    """EorEmployeeCostCalculationRequestBodyContainer

    :param data: data
    :type data: EorEmployeeCostCalculationRequestBody
    """

    def __init__(self, data: EorEmployeeCostCalculationRequestBody):
        self.data = self._define_object(data, EorEmployeeCostCalculationRequestBody)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class BankAccountToAdd(BaseModel):
    """BankAccountToAdd

    :param key: The key of the bank account properties.
    :type key: str
    :param value: The value for that key property.
    :type value: str
    """

    def __init__(self, key: str, value: str):
        self.key = key
        self.value = value



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .timesheet_shared_properties import TimesheetSharedProperties


@JsonMap({})
class TimesheetToCreateContainer(BaseModel):
    """TimesheetToCreateContainer

    :param data: Details of the timesheet to create. Both a client or a contractor may create a timesheet.
    :type data: TimesheetSharedProperties
    """

    def __init__(self, data: TimesheetSharedProperties):
        self.data = self._define_object(data, TimesheetSharedProperties)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .worker_document_download_link import WorkerDocumentDownloadLink


@JsonMap({})
class DownloadWorkerDocumentsByIdContainer(BaseModel):
    """DownloadWorkerDocumentsByIdContainer

    :param data: data
    :type data: WorkerDocumentDownloadLink
    """

    def __init__(self, data: WorkerDocumentDownloadLink):
        self.data = self._define_object(data, WorkerDocumentDownloadLink)



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class EorHolidaysRolloverType(Enum):
    """An enumeration representing different categories.

    :cvar EXPIRATION: "EXPIRATION"
    :vartype EXPIRATION: str
    :cvar PAY_OUT: "PAY_OUT"
    :vartype PAY_OUT: str
    :cvar UNLIMITED_ROLLOVER: "UNLIMITED_ROLLOVER"
    :vartype UNLIMITED_ROLLOVER: str
    :cvar LIMITED_ROLLOVER: "LIMITED_ROLLOVER"
    :vartype LIMITED_ROLLOVER: str
    :cvar UNSET: "UNSET"
    :vartype UNSET: str
    """

    EXPIRATION = "EXPIRATION"
    PAY_OUT = "PAY_OUT"
    UNLIMITED_ROLLOVER = "UNLIMITED_ROLLOVER"
    LIMITED_ROLLOVER = "LIMITED_ROLLOVER"
    UNSET = "UNSET"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, EorHolidaysRolloverType._member_map_.values())
        )



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class InvoiceStatusEnum(Enum):
    """An enumeration representing different categories.

    :cvar PENDING: "pending"
    :vartype PENDING: str
    :cvar PAID: "paid"
    :vartype PAID: str
    :cvar PROCESSING: "processing"
    :vartype PROCESSING: str
    :cvar CANCELED: "canceled"
    :vartype CANCELED: str
    :cvar SKIPPED: "skipped"
    :vartype SKIPPED: str
    :cvar FAILED: "failed"
    :vartype FAILED: str
    :cvar REFUNDED: "refunded"
    :vartype REFUNDED: str
    """

    PENDING = "pending"
    PAID = "paid"
    PROCESSING = "processing"
    CANCELED = "canceled"
    SKIPPED = "skipped"
    FAILED = "failed"
    REFUNDED = "refunded"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, InvoiceStatusEnum._member_map_.values()))



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .base import BaseModel
from .gp_client import GpClient
from .gp_contract_status_type_enum import GpContractStatusTypeEnum


@JsonMap({})
class EmployeeAddress2(BaseModel):
    """EmployeeAddress2

    :param street: Street and number.
    :type street: str
    :param city: City name.
    :type city: str
    :param state: State code., defaults to None
    :type state: str, optional
    :param zip: Zip code.
    :type zip: str
    :param country: Country code.
    :type country: str
    """

    def __init__(
        self, street: str, city: str, zip: str, country: str, state: str = None
    ):
        self.street = street
        self.city = city
        if state is not None:
            self.state = state
        self.zip = zip
        self.country = self._pattern_matching(country, "^[A-Z]{2}$", "country")


@JsonMap({})
class GpContractToCreateEmployee(BaseModel):
    """GpContractToCreateEmployee

    :param first_name: Employee's first name.
    :type first_name: str
    :param last_name: Employee's last name.
    :type last_name: str
    :param email: Worker's email address.
    :type email: str
    :param work_email: Worker's email address.
    :type work_email: str
    :param nationality: Employee's nationality., defaults to None
    :type nationality: str, optional
    :param employee_number: Employee's number., defaults to None
    :type employee_number: str, optional
    :param address: address
    :type address: EmployeeAddress2
    """

    def __init__(
        self,
        first_name: str,
        last_name: str,
        email: str,
        work_email: str,
        address: EmployeeAddress2,
        nationality: str = None,
        employee_number: str = None,
    ):
        self.first_name = first_name
        self.last_name = last_name
        self.email = email
        self.work_email = work_email
        if nationality is not None:
            self.nationality = nationality
        if employee_number is not None:
            self.employee_number = employee_number
        self.address = self._define_object(address, EmployeeAddress2)


@JsonMap({})
class EmploymentHolidays1(BaseModel):
    """EmploymentHolidays1

    :param allowance: Enter the number of holidays.
    :type allowance: float
    :param start_date: Short date in format ISO-8601 (YYYY-MM-DD). For example: 2022-12-31.
    :type start_date: str
    """

    def __init__(self, allowance: float, start_date: str):
        self.allowance = allowance
        self.start_date = start_date


@JsonMap({"type_": "type"})
class GpContractToCreateEmployment(BaseModel):
    """GpContractToCreateEmployment

    :param type_: Is it a full-time contract or a part-time contract?
    :type type_: GpContractStatusTypeEnum
    :param start_date: Short date in format ISO-8601 (YYYY-MM-DD). For example: 2022-12-31.
    :type start_date: str
    :param holidays: holidays
    :type holidays: EmploymentHolidays1
    """

    def __init__(
        self,
        type_: GpContractStatusTypeEnum,
        start_date: str,
        holidays: EmploymentHolidays1,
    ):
        self.type_ = self._enum_matching(
            type_, GpContractStatusTypeEnum.list(), "type_"
        )
        self.start_date = start_date
        self.holidays = self._define_object(holidays, EmploymentHolidays1)


class Scale(Enum):
    """An enumeration representing different categories.

    :cvar YEAR: "YEAR"
    :vartype YEAR: str
    :cvar MONTH: "MONTH"
    :vartype MONTH: str
    """

    YEAR = "YEAR"
    MONTH = "MONTH"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, Scale._member_map_.values()))


@JsonMap({})
class GpContractToCreateCompensationDetails(BaseModel):
    """GpContractToCreateCompensationDetails

    :param scale: Defines the scale at which the salary is paid.
    :type scale: Scale
    :param salary: Employee's gross annual/month salary.
    :type salary: float
    :param currency: currency, defaults to None
    :type currency: str, optional
    """

    def __init__(self, scale: Scale, salary: float, currency: str = None):
        self.scale = self._enum_matching(scale, Scale.list(), "scale")
        self.salary = salary
        if currency is not None:
            self.currency = self._pattern_matching(currency, "^[A-Z]{3}$", "currency")


@JsonMap({})
class GpContractToCreate(BaseModel):
    """GpContractToCreate

    :param employee: employee
    :type employee: GpContractToCreateEmployee
    :param employment: employment
    :type employment: GpContractToCreateEmployment
    :param job_title: Employee's job title.
    :type job_title: str
    :param client: client
    :type client: GpClient
    :param compensation_details: compensation_details
    :type compensation_details: GpContractToCreateCompensationDetails
    """

    def __init__(
        self,
        employee: GpContractToCreateEmployee,
        employment: GpContractToCreateEmployment,
        job_title: str,
        client: GpClient,
        compensation_details: GpContractToCreateCompensationDetails,
    ):
        self.employee = self._define_object(employee, GpContractToCreateEmployee)
        self.employment = self._define_object(employment, GpContractToCreateEmployment)
        self.job_title = job_title
        self.client = self._define_object(client, GpClient)
        self.compensation_details = self._define_object(
            compensation_details, GpContractToCreateCompensationDetails
        )



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class EmployeeAgreementDownloadObject(BaseModel):
    """EmployeeAgreementDownloadObject

    :param url: URL to requested employee agreement to download., defaults to None
    :type url: str, optional
    :param expires_at: Expiration date time of download URL., defaults to None
    :type expires_at: str, optional
    """

    def __init__(self, url: str = None, expires_at: str = None):
        if url is not None:
            self.url = url
        if expires_at is not None:
            self.expires_at = expires_at



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .legal_entity_type import LegalEntityType


@JsonMap({"type_": "type"})
class WorkerLegalEntity(BaseModel):
    """WorkerLegalEntity

    :param type_: type_
    :type type_: LegalEntityType
    :param personal_name: personal_name
    :type personal_name: str
    :param registration_number: registration_number
    :type registration_number: str
    """

    def __init__(
        self, type_: LegalEntityType, personal_name: str, registration_number: str
    ):
        self.type_ = self._enum_matching(type_, LegalEntityType.list(), "type_")
        self.personal_name = personal_name
        self.registration_number = registration_number



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .adjustment_to_update import AdjustmentToUpdate


@JsonMap({})
class AdjustmentToUpdateContainer(BaseModel):
    """AdjustmentToUpdateContainer

    :param data: Details of adjustment to update
    :type data: AdjustmentToUpdate
    """

    def __init__(self, data: AdjustmentToUpdate):
        self.data = self._define_object(data, AdjustmentToUpdate)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .pgo_task_reviews_to_create import PgoTaskReviewsToCreate


@JsonMap({})
class RequestBodyToCreatePgoTaskReviewsReviewsContainer(BaseModel):
    """RequestBodyToCreatePgoTaskReviewsReviewsContainer

    :param data: data
    :type data: PgoTaskReviewsToCreate
    """

    def __init__(self, data: PgoTaskReviewsToCreate):
        self.data = self._define_object(data, PgoTaskReviewsToCreate)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class UpdateWorkerWorkingLocation(BaseModel):
    """UpdateWorkerWorkingLocation

    :param working_location_id: Unique identifier of this resource.
    :type working_location_id: str
    """

    def __init__(self, working_location_id: str):
        self.working_location_id = working_location_id



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .adjustment_status_enum import AdjustmentStatusEnum


@JsonMap({"id_": "id", "file_type": "fileType"})
class File(BaseModel):
    """File

    :param id_: The unique identifier of the file, defaults to None
    :type id_: str, optional
    :param name: The name of the file, defaults to None
    :type name: str, optional
    :param file_type: The type of the file, defaults to None
    :type file_type: str, optional
    """

    def __init__(self, id_: str = None, name: str = None, file_type: str = None):
        if id_ is not None:
            self.id_ = id_
        if name is not None:
            self.name = name
        if file_type is not None:
            self.file_type = file_type


@JsonMap({"id_": "id"})
class PayrollAdjustment(BaseModel):
    """PayrollAdjustment

    :param id_: The unique identifier of the adjustment, defaults to None
    :type id_: str, optional
    :param contract_id: The identifier of the contract associated with the adjustment, defaults to None
    :type contract_id: str, optional
    :param amount: The amount of the adjustment, defaults to None
    :type amount: str, optional
    :param date_of_adjustment: The date of the adjustment, defaults to None
    :type date_of_adjustment: str, optional
    :param title: The title of the adjustment, defaults to None
    :type title: str, optional
    :param description: The description of the adjustment, defaults to None
    :type description: str, optional
    :param cycle_reference: The reference to the cycle associated with the adjustment, defaults to None
    :type cycle_reference: str, optional
    :param status: Adjustment status, defaults to None
    :type status: AdjustmentStatusEnum, optional
    :param adjustment_category_id: The identifier of the adjustment category associated with the adjustment, defaults to None
    :type adjustment_category_id: str, optional
    :param actual_start_cycle_date: The date of the actual start cycle date, defaults to None
    :type actual_start_cycle_date: str, optional
    :param actual_end_cycle_date: The date of the actual end cycle date, defaults to None
    :type actual_end_cycle_date: str, optional
    :param move_next_cycle: If an adjustments can belong to another payroll cycle, defaults to None
    :type move_next_cycle: bool, optional
    :param file: file, defaults to None
    :type file: File, optional
    :param created_at: The date and time when the adjustment was created, defaults to None
    :type created_at: str, optional
    :param updated_at: The date and time when the adjustment was last updated, defaults to None
    :type updated_at: str, optional
    """

    def __init__(
        self,
        id_: str = None,
        contract_id: str = None,
        amount: str = None,
        date_of_adjustment: str = None,
        title: str = None,
        description: str = None,
        cycle_reference: str = None,
        status: AdjustmentStatusEnum = None,
        adjustment_category_id: str = None,
        actual_start_cycle_date: str = None,
        actual_end_cycle_date: str = None,
        move_next_cycle: bool = None,
        file: File = None,
        created_at: str = None,
        updated_at: str = None,
    ):
        if id_ is not None:
            self.id_ = id_
        if contract_id is not None:
            self.contract_id = contract_id
        if amount is not None:
            self.amount = amount
        if date_of_adjustment is not None:
            self.date_of_adjustment = date_of_adjustment
        if title is not None:
            self.title = title
        if description is not None:
            self.description = description
        if cycle_reference is not None:
            self.cycle_reference = cycle_reference
        if status is not None:
            self.status = self._enum_matching(
                status, AdjustmentStatusEnum.list(), "status"
            )
        if adjustment_category_id is not None:
            self.adjustment_category_id = adjustment_category_id
        if actual_start_cycle_date is not None:
            self.actual_start_cycle_date = actual_start_cycle_date
        if actual_end_cycle_date is not None:
            self.actual_end_cycle_date = actual_end_cycle_date
        if move_next_cycle is not None:
            self.move_next_cycle = move_next_cycle
        if file is not None:
            self.file = self._define_object(file, File)
        if created_at is not None:
            self.created_at = created_at
        if updated_at is not None:
            self.updated_at = updated_at



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class UploadEmployeeComplianceDocumentContainerData(BaseModel):
    """UploadEmployeeComplianceDocumentContainerData

    :param success: A flag denoting if the file upload was successful., defaults to None
    :type success: bool, optional
    """

    def __init__(self, success: bool = None):
        if success is not None:
            self.success = success


@JsonMap({})
class UploadEmployeeComplianceDocumentContainer(BaseModel):
    """UploadEmployeeComplianceDocumentContainer

    :param data: data
    :type data: UploadEmployeeComplianceDocumentContainerData
    """

    def __init__(self, data: UploadEmployeeComplianceDocumentContainerData):
        self.data = self._define_object(
            data, UploadEmployeeComplianceDocumentContainerData
        )



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .work_statement_cycle_scale_enum import WorkStatementCycleScaleEnum
from .work_statement_cycle_end_type_enum import WorkStatementCycleEndTypeEnum
from .work_statement_payment_due_type_enum import WorkStatementPaymentDueTypeEnum


@JsonMap({})
class CompensationDetailsOfContractToCreateShared(BaseModel):
    """CompensationDetailsOfContractToCreateShared

    :param amount: Amount to be paid. This field can be excluded when creating a Pay-as-you-go task-based or Milestone contracts., defaults to None
    :type amount: float, optional
    :param currency_code: currency_code
    :type currency_code: str
    :param frequency: Scale of the invoice cycle.
    :type frequency: WorkStatementCycleScaleEnum
    :param cycle_end: Date invoice cycle ends.
    :type cycle_end: float
    :param cycle_end_type: Cycle end can be weekly 1-7 (Monday = 1) or monthly 1-31.
    :type cycle_end_type: WorkStatementCycleEndTypeEnum
    :param payment_due_type: Type of payment due. If the payment is due on the last day of month, enter 'WITHIN_MONTH'.
    :type payment_due_type: WorkStatementPaymentDueTypeEnum
    :param payment_due_days: payment_due_days
    :type payment_due_days: float
    :param pay_before_weekends: If the payment due is on a weekend, pay on Friday., defaults to None
    :type pay_before_weekends: bool, optional
    :param first_payment_date: Short date in format ISO-8601 (YYYY-MM-DD). For example 2022-12-31., defaults to None
    :type first_payment_date: str, optional
    :param first_payment: First payment amount., defaults to None
    :type first_payment: float, optional
    :param notice_period: Days before to notice the termination of contract for eather party., defaults to None
    :type notice_period: float, optional
    """

    def __init__(
        self,
        currency_code: str,
        frequency: WorkStatementCycleScaleEnum,
        cycle_end: float,
        cycle_end_type: WorkStatementCycleEndTypeEnum,
        payment_due_type: WorkStatementPaymentDueTypeEnum,
        payment_due_days: float,
        amount: float = None,
        pay_before_weekends: bool = None,
        first_payment_date: str = None,
        first_payment: float = None,
        notice_period: float = None,
    ):
        if amount is not None:
            self.amount = amount
        self.currency_code = self._pattern_matching(
            currency_code, "^[A-Z]{3}$", "currency_code"
        )
        self.frequency = self._enum_matching(
            frequency, WorkStatementCycleScaleEnum.list(), "frequency"
        )
        self.cycle_end = cycle_end
        self.cycle_end_type = self._enum_matching(
            cycle_end_type, WorkStatementCycleEndTypeEnum.list(), "cycle_end_type"
        )
        self.payment_due_type = self._enum_matching(
            payment_due_type, WorkStatementPaymentDueTypeEnum.list(), "payment_due_type"
        )
        self.payment_due_days = payment_due_days
        if pay_before_weekends is not None:
            self.pay_before_weekends = pay_before_weekends
        if first_payment_date is not None:
            self.first_payment_date = self._pattern_matching(
                first_payment_date, "^\d{4}-\d{2}-\d{2}$", "first_payment_date"
            )
        if first_payment is not None:
            self.first_payment = first_payment
        if notice_period is not None:
            self.notice_period = notice_period
